"""Agora WebRTC API client for server-to-client communication.

This module provides a client for interacting with the Agora WebRTC API endpoint
at `/api/v2/transpond/webrtc?v=2`. It handles request construction, API calls,
and response parsing to get edge server addresses and tickets for WebRTC connections.

The implementation is based on analysis of the Agora JavaScript SDK (agoraRTC_N.js)
to ensure compatibility and parity with client-side behavior.
"""

from dataclasses import dataclass
import hashlib
import json
import logging
from random import randint
import time
from types import TracebackType
from typing import Any

import aiohttp

# Service IDs for API requests (what you send in the request)
SERVICE_IDS = {
    "CHOOSE_SERVER": 11,  # Media gateway / WebSocket edge servers
    "CLOUD_PROXY": 18,
    "CLOUD_PROXY_5": 20,
    "CLOUD_PROXY_FALLBACK": 26,  # TURN servers
}

# Response flags (what you receive in response_body[].buffer.flag)
RESPONSE_FLAGS = {
    "CHOOSE_SERVER": 4096,  # Media gateway addresses
    "CLOUD_PROXY": 1048576,
    "CLOUD_PROXY_5": 4194304,
    "CLOUD_PROXY_FALLBACK": 4194310,  # TURN/proxy addresses
}


def derive_password(uid: int | str) -> str:
    """Derive TURN/STUN password using SHA-256 hash.

    Python equivalent of the JavaScript Ww function from agoraRTC_N.js:
    const Ww = async e => digest("SHA-256", jw(e)).hex()

    This is used when ENCRYPT_PROXY_USERNAME_AND_PSW feature flag is enabled.

    Args:
        uid: User ID (numeric or string)

    Returns:
        Hexadecimal string representation of SHA-256 hash

    """
    uid_str = str(uid)
    return hashlib.sha256(uid_str.encode("utf-8")).hexdigest()


@dataclass
class EdgeAddress:
    """Represents an edge server address."""

    ip: str
    port: int
    username: str | None = None
    credentials: str | None = None
    ticket: str | None = None
    fingerprint: str | None = None

    def to_dict(self) -> dict:
        """Convert to dictionary."""
        result = {"ip": self.ip, "port": self.port}
        if self.ticket:
            result["ticket"] = self.ticket
        if self.fingerprint:
            result["fingerprint"] = self.fingerprint
        return result


@dataclass
class ICEServer:
    """Represents an RTCIceServer configuration."""

    urls: str | list[str]
    username: str | None = None
    credential: str | None = None

    def to_dict(self) -> dict:
        """Convert to dictionary for RTCPeerConnection."""
        result = {"urls": self.urls}
        if self.username:
            result["username"] = self.username
        if self.credential:
            result["credential"] = self.credential
        return result


@dataclass
class AgoraResponse:
    """Parsed response from Agora WebRTC API.

    When requesting multiple service flags, contains separate entries for
    each flag in the responses dict.
    """

    code: int
    addresses: list[EdgeAddress]
    ticket: str
    uid: int
    cid: int
    cname: str
    server_ts: int
    detail: dict
    flag: int
    opid: int
    responses: dict | None = None  # Multi-flag responses: {flag: response_dict}

    @classmethod
    def from_api_response(cls, response_data: dict) -> "AgoraResponse":
        """Parse API response into AgoraResponse object.

        Handles both single and multiple service flag responses.

        Args:
            response_data: Raw response from `/api/v2/transpond/webrtc?v=2` endpoint

        Returns:
            Parsed AgoraResponse with extracted addresses and ticket

        """
        # Extract response body
        response_body = response_data.get("response_body", [])
        detail = response_data.get("detail", {})
        if not response_body:
            raise ValueError("No response_body in API response")

        _log = logging.getLogger(__name__)
        _log.debug("Agora API response body count: %d", len(response_body))

        # Parse all responses by flag
        responses_by_flag = {}
        first_buffer = None

        for response_item in response_body:
            buffer = response_item.get("buffer", {})
            code = buffer.get("code", -1)

            if code != 0:
                raise Exception(f"Agora API returned error code: {code}")

            flag = buffer.get("flag", 0)
            ticket = buffer.get("cert", "")
            edges_services = buffer.get("edges_services", [])
            detail = {**detail, **buffer.get("detail", {})}
            uid = buffer.get("uid", 0)

            _log.info(
                "Parsing response flag=%d, uid=%d, edges_count=%d",
                flag,
                uid,
                len(edges_services),
            )

            # Note: We intentionally ignore the 'detail' fields for credentials (detail.8/detail.4)
            # to match Agora SDK behavior which uses UID-derived credentials in secure contexts.
            # Using detail fields causes TURN 401 failures because the server expects UID hash.
            # The actual derivation happens below (lines 239-240).
            # Parse fingerprints from detail[19] (semicolon-separated list)
            # Each fingerprint corresponds to an edge address
            fingerprints = []
            if detail.get("19"):
                fingerprint_str = detail["19"]
                # Split by semicolon and strip whitespace
                fingerprints = [fp.strip() for fp in fingerprint_str.split(";") if fp.strip()]

            username = str(uid)
            credentials = derive_password(uid)

            addresses = [
                EdgeAddress(
                    ip=edge["ip"],
                    port=edge["port"],
                    username=username,
                    credentials=credentials,
                    ticket=ticket,
                    fingerprint=fingerprints[i] if i < len(fingerprints) else None,
                )
                for i, edge in enumerate(edges_services)
            ]

            # Store all responses with complete data
            responses_by_flag[flag] = {
                "code": code,
                "addresses": addresses,
                "ticket": ticket,
                "uid": buffer.get("uid", 0),
                "cid": buffer.get("cid", 0),
                "cname": buffer.get("cname", ""),
                "detail": detail,
                "flag": flag,
                "edges_services": edges_services,  # Preserve raw edges_services
            }

            # Use first response for primary fields
            if first_buffer is None:
                first_buffer = buffer

        if first_buffer is None:
            raise ValueError("No valid buffer in response_body")

        # Get the first flag's response data (already parsed with addresses)
        first_flag = first_buffer.get("flag", 0)
        first_response = responses_by_flag.get(4096, responses_by_flag.get(first_flag, {}))

        # Create response with primary fields from first buffer
        return cls(
            code=first_buffer.get("code", -1),
            addresses=first_response.get("addresses", []),  # Use already-created addresses
            ticket=first_buffer.get("cert", ""),
            uid=first_buffer.get("uid", 0),
            cid=first_buffer.get("cid", 0),
            cname=first_buffer.get("cname", ""),
            server_ts=response_data.get("enter_ts", int(time.time() * 1000)),
            detail=first_buffer.get("detail", {}),
            flag=first_flag,
            opid=response_data.get("opid", 0),
            responses=responses_by_flag if len(responses_by_flag) > 1 else None,
        )

    def get_ice_servers(self, use_all_turn_servers: bool = True, new_turn_mode: int = 4) -> list[ICEServer]:
        """Convert TURN addresses to ICE server configuration.

        Args:
            use_all_turn_servers: If True, includes all TURN servers. If False, only first one.
            new_turn_mode: TURN mode (1=udp, 2=tcp, 3=tls, 4=all, default: 4)

        Returns:
            List of ICEServer objects ready for RTCPeerConnection

        """
        _log = logging.getLogger(__name__)
        ice_servers = []

        # Get TURN addresses from flag 4194310
        turn_addresses = self.get_turn_addresses()

        if not turn_addresses:
            # Fallback to any available addresses
            turn_addresses = self.addresses
            _log.warning("No TURN addresses found with flag 4194310, using primary addresses")

        # Use all servers or just the first one
        addresses_to_use = turn_addresses if use_all_turn_servers else turn_addresses[:1]

        _log.info(
            "Creating ICE servers: use_all=%s, mode=%s, addr_count=%d",
            use_all_turn_servers,
            new_turn_mode,
            len(addresses_to_use),
        )

        for addr in addresses_to_use:
            _log.debug(
                "Processing TURN address: ip=%s, port=%d, username=%s, cred_len=%s",
                addr.ip,
                addr.port,
                addr.username,
                len(addr.credentials) if addr.credentials else 0,
            )

            # VALIDATION: Check credentials are present before creating ICE servers
            if not addr.username:
                _log.error(
                    "CRITICAL: TURN address %s:%d has empty username! This will cause 401 errors.",
                    addr.ip,
                    addr.port,
                )
            if not addr.credentials:
                _log.error(
                    "CRITICAL: TURN address %s:%d has empty credentials! This will cause 401 errors.",
                    addr.ip,
                    addr.port,
                )

            # Based on new_turn_mode (from agoraRTC_N.js:30764-30796)
            if new_turn_mode in [1, 4]:  # UDP
                ice_servers.append(
                    ICEServer(
                        urls=f"turn:{addr.ip}:3478?transport=udp",
                        username=addr.username,
                        credential=addr.credentials,
                    )
                )

            if new_turn_mode in [2, 4]:  # TCP
                ice_servers.append(
                    ICEServer(
                        urls=f"turn:{addr.ip}:3478?transport=tcp",
                        username=addr.username,
                        credential=addr.credentials,
                    )
                )

            if new_turn_mode in [3, 4]:  # TLS
                ice_servers.append(
                    ICEServer(
                        urls=f"turns:{addr.ip.replace('.', '-')}.edge.agora.io:443?transport=tcp",
                        username=addr.username,
                        credential=addr.credentials,
                    )
                )

        _log.info(
            "Created %d ICE server entries from %d addresses",
            len(ice_servers),
            len(addresses_to_use),
        )

        # SUMMARY: Log all created ICE servers for validation
        if ice_servers:
            _log.info("ICE Server Summary:")
            for i, server in enumerate(ice_servers):
                _log.info(
                    "  [%d] urls=%s, username=%s, cred_present=%s",
                    i,
                    server.urls,
                    server.username,
                    bool(server.credential),
                )
        else:
            _log.error("WARNING: No ICE servers were created! This will prevent TURN connections.")

        return ice_servers

    def get_turn_server_config(
        self,
        gateway_address: EdgeAddress | None = None,
        token: str | None = None,
        use_gateway: bool = True,
    ) -> dict:
        """Generate complete turnServer configuration matching Agora SDK format.

        Returns object with both 'servers' (from flag 4194310) and
        'serversFromGateway' (derived from connected gateway).

        Args:
            gateway_address: The EdgeAddress of currently connected gateway (for serversFromGateway)
            token: The channel token (for serversFromGateway password)
            use_gateway: Whether to include serversFromGateway

        Returns:
            Dict with 'mode', 'servers', and 'serversFromGateway' arrays

        """
        config = {"mode": "manual", "servers": [], "serversFromGateway": []}

        # Build 'servers' array from TURN addresses (flag 4194310)
        turn_addresses = self.get_turn_addresses()
        for addr in turn_addresses:
            config["servers"].append(
                {
                    "turnServerURL": addr.ip,
                    "tcpport": addr.port,
                    "udpport": addr.port,
                    "username": str(self.uid),
                    "password": derive_password(self.uid),
                    "forceturn": False,
                    "security": True,  # Always true for proxy fallback
                }
            )

        # Build 'serversFromGateway' from connected gateway
        if use_gateway and gateway_address and token:
            config["serversFromGateway"].append(
                {
                    "username": str(self.uid),
                    "password": token,  # Use JWT token, not hashed
                    "turnServerURL": gateway_address.ip,
                    "tcpport": gateway_address.port + 30,  # Gateway port + 30
                    "udpport": gateway_address.port + 30,
                    "forceturn": False,
                    # Note: no 'security' field for serversFromGateway
                }
            )

        return config

    def get_responses_by_flag(self, flag: int) -> dict | None:
        """Get response data for a specific service flag.

        Args:
            flag: Response flag (e.g., RESPONSE_FLAGS["CHOOSE_SERVER"])

        Returns:
            Response data for that flag, or None if not available

        """
        if not self.responses:
            return None
        return self.responses.get(flag)

    def get_gateway_addresses(self) -> list[EdgeAddress]:
        """Get WebSocket gateway addresses (flag 4096).

        Returns:
            List of gateway EdgeAddress objects for WebSocket connections

        """
        if self.responses:
            gateway_data = self.responses.get(RESPONSE_FLAGS["CHOOSE_SERVER"])
            if gateway_data:
                return gateway_data.get("addresses", [])
        # Fallback to primary addresses if flag 4096 is the main response
        if self.flag == RESPONSE_FLAGS["CHOOSE_SERVER"]:
            return self.addresses
        return []

    def get_turn_addresses(self) -> list[EdgeAddress]:
        """Get TURN server addresses (flag 4194310).

        Returns:
            List of TURN server EdgeAddress objects

        """
        if self.responses:
            turn_data = self.responses.get(RESPONSE_FLAGS["CLOUD_PROXY_FALLBACK"])
            if turn_data:
                return turn_data.get("addresses", [])
        # Fallback to primary addresses if flag 4194310 is the main response
        if self.flag == RESPONSE_FLAGS["CLOUD_PROXY_FALLBACK"]:
            return self.addresses
        return []

    def to_ap_response(self, flag: int | None = None) -> dict:
        """Format response data for websocket join_v3 ap_response field.

        Args:
            flag: Specific service flag to format. If None, uses primary response.

        Returns:
            Dictionary formatted for ap_response in join_v3 websocket call

        """
        # Get data for specific flag or use primary response
        if flag is not None and self.responses:
            response_data = self.responses.get(flag)
            if not response_data:
                raise ValueError(f"No response data for flag {flag}")
            return {
                "code": response_data["code"],
                "server_ts": self.server_ts,
                "uid": response_data["uid"],
                "cid": response_data["cid"],
                "cname": response_data["cname"],
                "detail": response_data["detail"],
                "flag": response_data["flag"],
                "opid": self.opid,
                "cert": response_data["ticket"],
                "ticket": response_data["ticket"],
            }
        # Use primary response data
        return {
            "code": self.code,
            "server_ts": self.server_ts,
            "uid": self.uid,
            "cid": self.cid,
            "cname": self.cname,
            "detail": self.detail,
            "flag": self.flag,
            "opid": self.opid,
            "cert": self.ticket,
            "ticket": self.ticket,
        }


class AgoraAPIClient:
    """Client for Agora WebRTC API.

    This client handles creating properly formatted requests to the Agora
    WebRTC server discovery endpoint and parsing responses.
    """

    # List of edge servers from Agora
    WEBCS_DOMAIN = [
        "webrtc2-ap-web-1.agora.io",
        "webrtc2-ap-web-2.agora.io",
        "webrtc2-ap-web-3.agora.io",
        "webrtc2-ap-web-4.agora.io",
    ]

    WEBCS_DOMAIN_BACKUP = [
        "webrtc2-ap-web-5.agora.io",
        "webrtc2-ap-web-6.agora.io",
    ]

    def __init__(self, session: aiohttp.ClientSession | None = None) -> None:
        """Initialize Agora API client.

        Args:
            session: Optional aiohttp ClientSession. If not provided, one will be
                    created for each request.

        """
        self.session = session
        self._own_session = session is None

    async def __aenter__(self):
        """Context manager entry."""
        return self

    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None:
        """Context manager exit - close session if we created it."""
        if self._own_session and self.session:
            await self.session.close()

    async def choose_server(
        self,
        app_id: str,
        token: str,
        channel_name: str,
        user_id: int,
        string_uid: str | None = None,
        role: int = 1,
        area_code: str = "CN,GLOBAL",
        service_flags: list[int] | None = None,
        sid: str | None = None,
        proxy_server: str | None = None,
    ) -> AgoraResponse:
        """Make a request to choose a server (URI 22).

        This is the initial "choose server" request that returns gateway addresses.

        Args:
            app_id: Agora application ID
            token: Authentication token
            channel_name: Channel name to join
            user_id: Numeric user ID
            string_uid: Optional string user ID (defaults to str(user_id))
            role: User role - 1 for host, 2 for audience (default: 2)
            area_code: Preferred area code (default: "CN,GLOBAL")
            service_flags: List of service flags (default: [CHOOSE_SERVER])
            sid: Session ID (generated if not provided)
            proxy_server: Optional HTTP proxy server URL

        Returns:
            AgoraResponse with edge server addresses and ticket

        Raises:
            Exception: If API call fails or returns error code

        """
        if string_uid is None:
            string_uid = str(user_id)

        if service_flags is None:
            service_flags = [11, 26]

        if sid is None:
            sid = str(randint(0, 2**31 - 1))

        # Build request payload
        request_payload = self._build_request_payload(
            app_id=app_id,
            token=token,
            channel_name=channel_name,
            user_id=user_id,
            string_uid=string_uid,
            role=role,
            area_code=area_code,
            service_flags=service_flags,
            sid=sid,
            uri=22,  # Choose server operation
        )
        _log = logging.getLogger(__name__)
        _log.debug("Agora choose_server request payload: %s", request_payload)
        # Make API call
        response = await self._make_api_call(request_payload, proxy_server=proxy_server)

        # Parse response
        return AgoraResponse.from_api_response(response)

    async def update_ticket(
        self,
        app_id: str,
        token: str,
        channel_name: str,
        user_id: int,
        string_uid: str | None = None,
        edge_addresses: list[dict] | None = None,
        sid: str | None = None,
        service_flags: list[int] | None = None,
        proxy_server: str | None = None,
    ) -> AgoraResponse:
        """Make a request to update ticket (URI 28).

        This is a follow-up request after initial server selection to refresh
        the connection ticket.

        Args:
            app_id: Agora application ID
            token: Authentication token
            channel_name: Channel name
            user_id: Numeric user ID
            string_uid: Optional string user ID
            edge_addresses: List of edge server addresses from previous response
            sid: Session ID
            service_flags: List of service flags
            proxy_server: Optional HTTP proxy server URL

        Returns:
            AgoraResponse with updated ticket

        Raises:
            Exception: If API call fails or returns error code

        """
        if string_uid is None:
            string_uid = str(user_id)

        if service_flags is None:
            service_flags = [SERVICE_IDS["CHOOSE_SERVER"]]

        if edge_addresses is None:
            edge_addresses = []

        # Build request payload
        request_payload = self._build_request_payload(
            app_id=app_id,
            token=token,
            channel_name=channel_name,
            user_id=user_id,
            string_uid=string_uid,
            edge_addresses=edge_addresses,
            service_flags=service_flags,
            sid=sid,
            uri=28,  # Ticket update operation
        )

        _log = logging.getLogger(__name__)
        _log.debug("Agora update_ticket request payload: %s", request_payload)

        # Make API call
        response = await self._make_api_call(request_payload, proxy_server=proxy_server)

        # Parse response
        return AgoraResponse.from_api_response(response)

    @staticmethod
    def merge_objects(*objects):
        """Merge multiple dictionaries, filtering out None values.

        Python equivalent of the JavaScript mF function used in Agora SDK.
        Merges objects left to right, skipping None/undefined values.

        Args:
            *objects: Variable number of dictionaries to merge

        Returns:
            Merged dictionary with None values filtered out

        """
        result = {}
        for obj in objects:
            if obj is not None:
                # Merge object, filtering out None values (equivalent to undefined in JS)
                for key, value in obj.items():
                    if value is not None:
                        result[key] = value
        return result

    def _build_request_payload(
        self,
        app_id: str,
        token: str,
        channel_name: str,
        user_id: int,
        string_uid: str,
        service_flags: list[int],
        sid: str,
        uri: int,
        role: int = 1,
        area_code: str = "CN,GLOBAL",
        edge_addresses: list[dict] | None = None,
    ) -> dict:
        """Build the request payload for Agora API.

        Args:
            app_id: Application ID
            token: Auth token
            channel_name: Channel name
            user_id: Numeric user ID
            string_uid: String user ID
            service_flags: Service flags
            sid: Session ID
            uri: Operation URI (22 for choose server, 28 for update ticket)
            role: User role (default: 2 for audience)
            area_code: Area code (default: CN,GLOBAL)
            edge_addresses: Edge addresses for ticket update

        Returns:
            Properly formatted request payload

        """
        client_ts = int(time.time() * 1000)
        opid = randint(0, 10**12 - 1)
        ap_rtm = None
        # Build detail field - matches JavaScript SDK pattern
        # mF(mF(mF({ 6: stringUid, 11: t, 12: USE_NEW_TOKEN ? "1" : undefined },
        #           r ? { 17: r } : {}), {}, { 22: t }, ...)
        # if use new token add "12": "1"
        # "6": string_uid,
        detail = self.merge_objects(
            {"11": area_code},
            {"17": str(role)} if role else {},
            {"22": area_code},
            {"26": "RTM2"} if ap_rtm else {},
        )

        _log = logging.getLogger(__name__)
        _log.debug("Built detail field for request: %s", detail)
        # Build buffer
        buffer = {
            "cname": channel_name,
            "detail": detail,
            "key": token,
            "service_ids": service_flags,
            "uid": user_id,
        }

        # For ticket update, include edge services
        if edge_addresses:
            buffer["edges_services"] = edge_addresses

        request_payload = {
            "appid": app_id,
            "client_ts": client_ts,
            "opid": opid,
            "sid": sid,
            "request_bodies": [
                {
                    "uri": uri,
                    "buffer": buffer,
                }
            ],
        }

        return request_payload

    async def _make_api_call(self, request_payload: dict, proxy_server: str | None = None) -> dict:
        """Make the actual HTTP API call to Agora endpoint.

        Args:
            request_payload: Request payload dictionary
            proxy_server: Optional HTTP proxy URL

        Returns:
            Parsed JSON response from API

        Raises:
            Exception: If request fails or response is invalid

        """
        session = self.session
        should_close = False

        if session is None:
            session = aiohttp.ClientSession()
            should_close = True

        try:
            # Try primary servers
            for domain in self.WEBCS_DOMAIN:
                try:
                    response = await self._call_endpoint(session, domain, request_payload, proxy_server)
                    return response
                except (TimeoutError, aiohttp.ClientError, Exception):
                    continue

            # Fall back to backup servers
            for domain in self.WEBCS_DOMAIN_BACKUP:
                try:
                    response = await self._call_endpoint(session, domain, request_payload, proxy_server)
                    return response
                except (TimeoutError, aiohttp.ClientError, Exception):
                    continue

            raise Exception("All Agora API servers failed to respond")

        finally:
            if should_close:
                await session.close()

    async def _call_endpoint(
        self,
        session: aiohttp.ClientSession,
        domain: str,
        request_payload: dict,
        proxy_server: str | None = None,
    ) -> Any:
        """Call a single Agora endpoint.

        Args:
            session: aiohttp session
            domain: Server domain
            request_payload: Request payload
            proxy_server: Optional proxy URL

        Returns:
            Parsed JSON response

        """
        url = f"https://{domain}/api/v2/transpond/webrtc?v=2"

        if proxy_server:
            url = f"https://{proxy_server}/ap/?url={domain}/api/v2/transpond/webrtc?v=2"

        # Create FormData with JSON payload
        form_data = aiohttp.FormData()
        form_data.add_field("request", json.dumps(request_payload), content_type="application/json")

        async with session.post(
            url,
            data=form_data,
            timeout=aiohttp.ClientTimeout(total=10),
            ssl=False,  # Note: In production, verify SSL certificates
        ) as resp:
            if resp.status != 200:
                raise Exception(f"HTTP {resp.status}: {await resp.text()}")

            return await resp.json()



"""Manage state from notifications into MowingDevice."""

from collections.abc import Awaitable, Callable
from datetime import UTC, datetime
import logging
from typing import Any

import betterproto2
from shapely import Point

from pymammotion.data.model.device import MowingDevice
from pymammotion.data.model.device_info import SideLight
from pymammotion.data.model.generate_geojson import GeojsonGenerator
from pymammotion.data.model.hash_list import (
    AreaHashNameList,
    MowPath,
    NavGetCommData,
    NavGetHashListData,
    Plan,
    SvgMessage,
)
from pymammotion.data.model.location import Dock, LocationPoint
from pymammotion.data.model.work import CurrentTaskSettings
from pymammotion.data.mqtt.event import ThingEventMessage
from pymammotion.data.mqtt.properties import ThingPropertiesMessage
from pymammotion.data.mqtt.status import ThingStatusMessage
from pymammotion.event.event import DataEvent
from pymammotion.proto import (
    AppGetAllAreaHashName,
    AppGetCutterWorkMode,
    AppSetCutterWorkMode,
    CoverPathUploadT,
    DeviceFwInfo,
    DeviceProductTypeInfoT,
    DrvDevInfoResp,
    DrvDevInfoResult,
    Getlamprsp,
    GetNetworkInfoRsp,
    LubaMsg,
    NavGetCommDataAck,
    NavGetHashListAck,
    NavPlanJobSet,
    NavReqCoverPath,
    NavSysParamMsg,
    NavUnableTimeSet,
    SvgMessageAckT,
    TimeCtrlLight,
    WifiIotStatusReport,
)
from pymammotion.utility.map import CoordinateConverter

logger = logging.getLogger(__name__)


class MowerStateManager:
    """Manage state."""

    def __init__(self, device: MowingDevice) -> None:
        """Initialize state manager with a device."""
        self._device: MowingDevice = device
        self.last_updated_at = datetime.now(UTC)
        self.cloud_gethash_ack_callback: Callable[[NavGetHashListAck], Awaitable[None]] | None = None
        self.cloud_get_commondata_ack_callback: (
            Callable[[NavGetCommDataAck | SvgMessageAckT], Awaitable[None]] | None
        ) = None
        self.cloud_on_notification_callback = DataEvent()
        self.cloud_queue_command_callback = DataEvent()

        self.cloud_get_plan_callback: Callable[[NavPlanJobSet], Awaitable[None]] | None = None
        self.ble_gethash_ack_callback: Callable[[NavGetHashListAck], Awaitable[None]] | None = None
        self.ble_get_commondata_ack_callback: Callable[[NavGetCommDataAck | SvgMessageAckT], Awaitable[None]] | None = (
            None
        )
        self.ble_get_plan_callback: Callable[[NavPlanJobSet], Awaitable[None]] | None = None
        self.ble_on_notification_callback = DataEvent()
        self.ble_queue_command_callback = DataEvent()

        self.properties_callback = DataEvent()
        self.status_callback = DataEvent()
        self.device_event_callback = DataEvent()

    def get_device(self) -> MowingDevice:
        """Get device."""
        return self._device

    def set_device(self, device: MowingDevice) -> None:
        """Set device."""
        self._device = device

    async def properties(self, thing_properties: ThingPropertiesMessage) -> None:
        """Update device properties and invoke callback."""
        # TODO update device based off thing properties
        self._device.mqtt_properties = thing_properties
        await self.on_properties_callback(thing_properties)

    async def status(self, thing_status: ThingStatusMessage) -> None:
        """Update device status and invoke callback."""
        if not self._device.online:
            self._device.online = True
        self._device.status_properties = thing_status
        if self._device.mower_state.product_key == "":
            self._device.mower_state.product_key = thing_status.params.product_key
        await self.on_status_callback(thing_status)

    async def device_event(self, device_event: ThingEventMessage) -> None:
        """Sets MQTT event and calls callback."""
        self._device.mqtt_device_event = device_event
        await self.on_device_event_callback(device_event)

    @property
    def online(self) -> bool:
        """Return online status."""
        return self._device.online

    @online.setter
    def online(self, value: bool) -> None:
        """Set online status."""
        self._device.online = value

    async def gethash_ack_callback(self, msg: NavGetHashListAck) -> None:
        """Dispatch hash list acknowledgment to available callback."""
        if self.cloud_gethash_ack_callback:
            await self.cloud_gethash_ack_callback(msg)
        elif self.ble_gethash_ack_callback:
            await self.ble_gethash_ack_callback(msg)

    async def on_notification_callback(self, res: tuple[str, Any | None]) -> None:
        """Dispatch notification to available callback."""
        if self.cloud_on_notification_callback:
            await self.cloud_on_notification_callback.data_event(res)
        elif self.ble_on_notification_callback:
            await self.ble_on_notification_callback.data_event(res)

    async def on_properties_callback(self, thing_properties: ThingPropertiesMessage) -> None:
        """Call properties callback if it exists."""
        if self.properties_callback:
            await self.properties_callback.data_event(thing_properties)

    async def on_status_callback(self, thing_status: ThingStatusMessage) -> None:
        """Execute the status callback if it is set."""
        if self.status_callback:
            await self.status_callback.data_event(thing_status)

    async def on_device_event_callback(self, device_event: ThingEventMessage) -> None:
        """Executes the event callback if it is set."""
        if self.device_event_callback:
            await self.device_event_callback.data_event(device_event)

    async def get_commondata_ack_callback(self, comm_data: NavGetCommDataAck | SvgMessageAckT) -> None:
        """Asynchronously calls the appropriate callback based on available handlers."""
        if self.cloud_get_commondata_ack_callback:
            await self.cloud_get_commondata_ack_callback(comm_data)
        elif self.ble_get_commondata_ack_callback:
            await self.ble_get_commondata_ack_callback(comm_data)

    async def get_plan_callback(self, planjob: NavPlanJobSet) -> None:
        """Dispatch plan job to available callback."""
        if self.cloud_get_plan_callback:
            await self.cloud_get_plan_callback(planjob)
        elif self.ble_get_plan_callback:
            await self.ble_get_plan_callback(planjob)

    async def queue_command_callback(self, **kwargs: Any) -> None:
        """Queue command to available callback."""
        if self.cloud_queue_command_callback:
            await self.cloud_queue_command_callback.data_event(**kwargs)
        elif self.ble_queue_command_callback:
            await self.ble_queue_command_callback.data_event(**kwargs)

    async def notification(self, message: LubaMsg) -> None:
        """Handle protobuf notifications."""
        res = betterproto2.which_one_of(message, "LubaSubMsg")
        self.last_updated_at = datetime.now(UTC)
        # additional catch all if we don't get a status update
        if not self._device.online:
            self._device.online = True

        match res[0]:
            case "nav":
                await self._update_nav_data(message)
            case "sys":
                self._update_sys_data(message)
            case "driver":
                self._update_driver_data(message)
            case "net":
                self._update_net_data(message)
            case "mul":
                self._update_mul_data(message)
            case "ota":
                self._update_ota_data(message)

        await self.on_notification_callback(res)

    async def _update_nav_data(self, message: LubaMsg) -> None:
        """Update nav data."""
        nav_msg = betterproto2.which_one_of(message.nav, "SubNavMsg")
        match nav_msg[0]:
            case "toapp_gethash_ack":
                hashlist_ack: NavGetHashListAck = nav_msg[1]
                self._device.map.update_root_hash_list(
                    NavGetHashListData.from_dict(hashlist_ack.to_dict(casing=betterproto2.Casing.SNAKE))
                )
                await self.gethash_ack_callback(nav_msg[1])
            case "toapp_get_commondata_ack":
                common_data: NavGetCommDataAck = nav_msg[1]
                updated = self._device.map.update(
                    NavGetCommData.from_dict(common_data.to_dict(casing=betterproto2.Casing.SNAKE))
                )
                if updated:
                    if len(self._device.map.missing_hashlist(0)) == 0:
                        self.generate_geojson(self._device.location.RTK, self._device.location.dock)

                    await self.get_commondata_ack_callback(common_data)
            case "cover_path_upload":
                mow_path: CoverPathUploadT = nav_msg[1]
                self._device.map.update_mow_path(MowPath.from_dict(mow_path.to_dict(casing=betterproto2.Casing.SNAKE)))
                if len(self._device.map.find_missing_mow_path_frames()) == 0:
                    self.generate_mowing_geojson(self._device.location.RTK)

            case "todev_planjob_set":
                planjob: NavPlanJobSet = nav_msg[1]
                self._device.map.update_plan(Plan.from_dict(planjob.to_dict(casing=betterproto2.Casing.SNAKE)))
                await self.get_plan_callback(planjob)

            case "toapp_svg_msg":
                common_svg_data: SvgMessageAckT = nav_msg[1]
                updated = self._device.map.update(
                    SvgMessage.from_dict(common_svg_data.to_dict(casing=betterproto2.Casing.SNAKE))
                )
                if updated:
                    await self.get_commondata_ack_callback(common_svg_data)

            case "toapp_all_hash_name":
                hash_names: AppGetAllAreaHashName = nav_msg[1]
                converted_list = [AreaHashNameList(name=item.name, hash=item.hash) for item in hash_names.hashnames]
                self._device.map.area_name = converted_list

            case "bidire_reqconver_path":
                work_settings: NavReqCoverPath = nav_msg[1]

                current_task = CurrentTaskSettings.from_dict(work_settings.to_dict(casing=betterproto2.Casing.SNAKE))

                if current_task.path_hash == 0:
                    self._device.map.current_mow_path = {}

                if work_settings.sub_cmd == 0:
                    await self.queue_command_callback(key="get_all_boundary_hash_list", sub_cmd=3)

                self._device.work = current_task

            case "nav_sys_param_cmd":
                settings: NavSysParamMsg = nav_msg[1]
                match settings.id:
                    case 3:
                        self._device.mower_state.rain_detection = bool(settings.context)
                    case 6:
                        self._device.mower_state.turning_mode = settings.context
                    case 7:
                        self._device.mower_state.traversal_mode = settings.context
            case "todev_unable_time_set":
                nav_non_work_time: NavUnableTimeSet = nav_msg[1]
                self._device.non_work_hours.non_work_sub_cmd = nav_non_work_time.sub_cmd
                self._device.non_work_hours.start_time = nav_non_work_time.unable_start_time
                self._device.non_work_hours.end_time = nav_non_work_time.unable_end_time

    def _update_sys_data(self, message) -> None:
        """Update system."""
        sys_msg = betterproto2.which_one_of(message.sys, "SubSysMsg")
        match sys_msg[0]:
            case "system_update_buf":
                self._device.buffer(sys_msg[1])
            case "toapp_report_data":
                self._device.update_report_data(sys_msg[1])
            case "mow_to_app_info":
                self._device.mow_info(sys_msg[1])
            case "system_tard_state_tunnel":
                self._device.run_state_update(sys_msg[1])
            case "todev_time_ctrl_light":
                ctrl_light: TimeCtrlLight = sys_msg[1]
                side_led: SideLight = SideLight.from_dict(ctrl_light.to_dict(casing=betterproto2.Casing.SNAKE))
                self._device.mower_state.side_led = side_led
            case "device_product_type_info":
                device_product_type: DeviceProductTypeInfoT = sys_msg[1]
                if device_product_type.main_product_type != "" or device_product_type.sub_product_type != "":
                    self._device.mower_state.model_id = device_product_type.main_product_type
                    self._device.mower_state.sub_model_id = device_product_type.sub_product_type
            case "toapp_dev_fw_info":
                device_fw_info: DeviceFwInfo = sys_msg[1]
                self._device.device_firmwares.device_version = device_fw_info.version
                self._device.mower_state.swversion = device_fw_info.version

    def _update_driver_data(self, message) -> None:
        """Update driver data."""
        driver_msg = betterproto2.which_one_of(message.driver, "SubDrvMsg")
        match driver_msg[0]:
            case "current_cutter_mode":
                cutter_work_mode: AppGetCutterWorkMode = driver_msg[1]
                self._device.mower_state.cutter_mode = cutter_work_mode.current_cutter_mode
                self._device.mower_state.cutter_rpm = cutter_work_mode.current_cutter_rpm
            case "cutter_mode_ctrl_by_hand":
                cutter_work_mode_set: AppSetCutterWorkMode = driver_msg[1]
                self._device.mower_state.cutter_mode = cutter_work_mode_set.cutter_mode

    def _update_net_data(self, message) -> None:
        """Update network data."""
        net_msg = betterproto2.which_one_of(message.net, "NetSubType")
        match net_msg[0]:
            case "toapp_wifi_iot_status":
                wifi_iot_status: WifiIotStatusReport = net_msg[1]
                self._device.mower_state.product_key = wifi_iot_status.productkey
            case "toapp_devinfo_resp":
                toapp_devinfo_resp: DrvDevInfoResp = net_msg[1]
                for resp in toapp_devinfo_resp.resp_ids:
                    if resp.res == DrvDevInfoResult.DRV_RESULT_SUC and resp.id == 1 and resp.type == 6:
                        self._device.mower_state.swversion = resp.info
                        self._device.device_firmwares.device_version = resp.info
            case "toapp_networkinfo_rsp":
                get_network_info_resp: GetNetworkInfoRsp = net_msg[1]
                self._device.mower_state.wifi_mac = get_network_info_resp.wifi_mac

    def _update_mul_data(self, message) -> None:
        """Media and video states."""
        mul_msg = betterproto2.which_one_of(message.mul, "SubMul")
        match mul_msg[0]:
            case "get_lamp_rsp":
                lamp_resp: Getlamprsp = mul_msg[1]
                self._device.mower_state.lamp_info.lamp_bright = lamp_resp.lamp_bright
                if lamp_resp.get_ids in (1126, 1127):
                    self._device.mower_state.lamp_info.lamp_bright = lamp_resp.lamp_bright
                    self._device.mower_state.lamp_info.manual_light = bool(lamp_resp.lamp_manual_ctrl.value) or bool(
                        lamp_resp.lamp_bright
                    )
                if lamp_resp.get_ids == 1123:
                    self._device.mower_state.lamp_info.lamp_bright = lamp_resp.lamp_bright
                    self._device.mower_state.lamp_info.night_light = bool(lamp_resp.lamp_ctrl.value) or bool(
                        lamp_resp.lamp_bright
                    )

    def _update_ota_data(self, message) -> None:
        """Update OTA data."""

    def generate_geojson(self, rtk: LocationPoint, dock: Dock) -> Any:
        """Generate geojson from frames."""
        coordinator_converter = CoordinateConverter(rtk.latitude, rtk.longitude)
        RTK_real_loc = coordinator_converter.enu_to_lla(0, 0)

        dock_location = coordinator_converter.enu_to_lla(dock.latitude, dock.longitude)
        dock_rotation = coordinator_converter.get_transform_yaw_with_yaw(dock.rotation) + 180

        self._device.map.generated_geojson = GeojsonGenerator.generate_geojson(
            self._device.map,
            Point(RTK_real_loc.latitude, RTK_real_loc.longitude),
            Point(dock_location.latitude, dock_location.longitude),
            int(dock_rotation),
        )

        return self._device.map.generated_geojson

    def generate_mowing_geojson(self, rtk: LocationPoint) -> Any:
        """Generate geojson from frames."""
        coordinator_converter = CoordinateConverter(rtk.latitude, rtk.longitude)
        RTK_real_loc = coordinator_converter.enu_to_lla(0, 0)

        self._device.map.generated_mow_path_geojson = GeojsonGenerator.generate_mow_path_geojson(
            self._device.map,
            Point(RTK_real_loc.latitude, RTK_real_loc.longitude),
        )

        return self._device.map.generated_mow_path_geojson



# === sendOrderMsg_Video ===
from abc import ABC
import time

from pymammotion.mammotion.commands.abstract_message import AbstractMessage
from pymammotion.proto import LubaMsg, MsgAttr, MsgCmdType, MsgDevice, MulCameraPosition, MulSetVideo, SocMul
from pymammotion.utility.device_type import DeviceType


class MessageVideo(AbstractMessage, ABC):
    def send_order_msg_video(self, mul: SocMul):
        luba_msg = LubaMsg(
            msgtype=MsgCmdType.MUL,
            msgattr=MsgAttr.REQ,
            sender=MsgDevice.DEV_MOBILEAPP,
            rcver=self.get_msg_device(MsgCmdType.MUL, MsgDevice.SOC_MODULE_MULTIMEDIA),
            mul=mul,
            seqs=self.seqs.increment_and_get() & 255,
            version=1,
            subtype=self.user_account,
            timestamp=round(time.time() * 1000),
        )

        return luba_msg.SerializeToString()

    def device_agora_join_channel_with_position(self, enter_state: int):
        position = (
            MulCameraPosition.ALL
            if DeviceType.value_of_str(self.get_device_name()).get_value() == DeviceType.LUBA_YUKA.get_value()
            else MulCameraPosition.LEFT
        )
        soc_mul = SocMul(set_video=MulSetVideo(position=position, vi_switch=enter_state))
        return self.send_order_msg_video(soc_mul)



from dataclasses import dataclass, field

from mashumaro.mixins.orjson import DataClassORJSONMixin

from pymammotion.http.model.http import ErrorInfo


@dataclass
class DeviceErrors(DataClassORJSONMixin):
    err_code_list: list[int] = field(default_factory=list)
    err_code_list_time: list[int] = field(default_factory=list)
    error_codes: dict[str, ErrorInfo] = field(default_factory=dict)



import struct


class MurMurHashUtil:
    MASK_32 = 0xFFFFFFFF
    MULTIPLIER = 1540483477

    @staticmethod
    def get_unsigned_int(i: int) -> int:
        """Convert signed int to unsigned (32-bit)"""
        return i & MurMurHashUtil.MASK_32

    @staticmethod
    def hash(data: bytes) -> int:
        """MurmurHash2 64-bit implementation"""
        pos = 0
        data_len = len(data)

        remaining = data_len ^ 97
        j = 0

        # Process 8 bytes at a time
        while (data_len - pos) >= 8:
            val1 = struct.unpack_from("<i", data, pos)[0]
            pos += 4

            unsigned_int_1 = MurMurHashUtil.get_unsigned_int(val1)
            temp1 = (unsigned_int_1 * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32
            temp2 = (temp1 ^ (temp1 >> 24)) & MurMurHashUtil.MASK_32
            temp3 = (temp2 * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32

            remaining = ((remaining * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32) ^ temp3
            remaining = remaining & MurMurHashUtil.MASK_32

            val2 = struct.unpack_from("<i", data, pos)[0]
            pos += 4

            unsigned_int_2 = MurMurHashUtil.get_unsigned_int(val2)
            temp1 = (unsigned_int_2 * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32
            temp2 = (temp1 ^ (temp1 >> 24)) & MurMurHashUtil.MASK_32
            temp3 = (temp2 * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32

            j = ((j * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32) ^ temp3
            j = j & MurMurHashUtil.MASK_32

        # Process remaining 4 bytes if available
        if (data_len - pos) >= 4:
            val = struct.unpack_from("<i", data, pos)[0]
            pos += 4

            unsigned_int_3 = MurMurHashUtil.get_unsigned_int(val)
            temp1 = (unsigned_int_3 * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32
            temp2 = (temp1 ^ (temp1 >> 24)) & MurMurHashUtil.MASK_32
            temp3 = (temp2 * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32

            remaining = ((remaining * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32) ^ temp3
            remaining = remaining & MurMurHashUtil.MASK_32

        # Process tail bytes (1-3 bytes)
        bytes_remaining = data_len - pos

        if bytes_remaining == 1:
            byte_val = data[pos] if data[pos] < 128 else data[pos] - 256
            j = (j ^ (MurMurHashUtil.get_unsigned_int(byte_val) & 255)) & MurMurHashUtil.MASK_32
            j = (j * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32

        elif bytes_remaining == 2:
            byte_val1 = data[pos + 1] if data[pos + 1] < 128 else data[pos + 1] - 256
            j = (j ^ ((MurMurHashUtil.get_unsigned_int(byte_val1) & 255) << 8)) & MurMurHashUtil.MASK_32

            byte_val0 = data[pos] if data[pos] < 128 else data[pos] - 256
            j = (j ^ (MurMurHashUtil.get_unsigned_int(byte_val0) & 255)) & MurMurHashUtil.MASK_32
            j = (j * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32

        elif bytes_remaining == 3:
            byte_val2 = data[pos + 2] if data[pos + 2] < 128 else data[pos + 2] - 256
            j = (j ^ ((MurMurHashUtil.get_unsigned_int(byte_val2) & 255) << 16)) & MurMurHashUtil.MASK_32

            byte_val1 = data[pos + 1] if data[pos + 1] < 128 else data[pos + 1] - 256
            j = (j ^ ((MurMurHashUtil.get_unsigned_int(byte_val1) & 255) << 8)) & MurMurHashUtil.MASK_32

            byte_val0 = data[pos] if data[pos] < 128 else data[pos] - 256
            j = (j ^ (MurMurHashUtil.get_unsigned_int(byte_val0) & 255)) & MurMurHashUtil.MASK_32
            j = (j * MurMurHashUtil.MULTIPLIER) & MurMurHashUtil.MASK_32

        # Final avalanche
        j4 = MurMurHashUtil.MULTIPLIER

        j5 = remaining ^ (j >> 18)
        j5 = (j5 & MurMurHashUtil.MASK_32) * j4
        j5 = j5 & MurMurHashUtil.MASK_32

        j6 = j ^ (j5 >> 22)
        j6 = (j6 & MurMurHashUtil.MASK_32) * j4
        j6 = j6 & MurMurHashUtil.MASK_32

        j7 = j5 ^ (j6 >> 17)
        j7 = (j7 & MurMurHashUtil.MASK_32) * j4
        j7 = j7 & MurMurHashUtil.MASK_32

        # Combine high and low parts
        j8 = j7 << 32

        low = j6 ^ (j7 >> 19)
        low = (low & MurMurHashUtil.MASK_32) * j4
        low = low & MurMurHashUtil.MASK_32

        result = j8 | low

        # Convert to signed 64-bit
        if result > 0x7FFFFFFFFFFFFFFF:
            result = result - 0x10000000000000000

        return result

    @staticmethod
    def hash_string(s: str) -> int:
        """Hash a string using UTF-8 encoding"""
        return MurMurHashUtil.hash(s.encode("utf-8"))

    @staticmethod
    def read_unsigned_long(value: int) -> int:
        """Convert to unsigned by masking with Long.MAX_VALUE"""
        return value & 0x7FFFFFFFFFFFFFFF

    @staticmethod
    def hash_unsigned(data: str | bytes) -> int:
        """Get unsigned hash value
        Can accept bytes or string
        """
        if isinstance(data, str):
            hash_val = MurMurHashUtil.hash_string(data)
        else:
            hash_val = MurMurHashUtil.hash(data)

        return MurMurHashUtil.read_unsigned_long(hash_val)

    @staticmethod
    def long_to_bytes(value: int) -> bytes:
        """Convert long to bytes exactly as Java does:
        1. Pack as big-endian (ByteBuffer default)
        2. Reverse all bytes
        """
        if value < 0:
            value = value & 0xFFFFFFFFFFFFFFFF

        big_endian = struct.pack(">Q", value)
        return big_endian[::-1]

    @staticmethod
    def hash_unsigned_list(values: list[int]) -> int:
        """Hash a list of long values"""
        data = b""

        for val in values:
            data += MurMurHashUtil.long_to_bytes(val)

        hash_val = MurMurHashUtil.hash(data)
        return MurMurHashUtil.read_unsigned_long(hash_val)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pymammotion/proto/mctrl_sys.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from pymammotion.proto import dev_net_pb2 as pymammotion_dot_proto_dot_dev__net__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!pymammotion/proto/mctrl_sys.proto\x1a\x1fpymammotion/proto/dev_net.proto\"\x1a\n\x08SysBatUp\x12\x0e\n\x06\x62\x61tVal\x18\x01 \x01(\x05\"Z\n\x0cSysWorkState\x12\x13\n\x0b\x64\x65viceState\x18\x01 \x01(\x05\x12\x13\n\x0b\x63hargeState\x18\x02 \x01(\x05\x12\x0e\n\x06\x63mHash\x18\x03 \x01(\x03\x12\x10\n\x08pathHash\x18\x04 \x01(\x03\"5\n\x0eSysSetTimeZone\x12\x11\n\ttimeStamp\x18\x01 \x01(\x05\x12\x10\n\x08timeArea\x18\x02 \x01(\x05\"\x9e\x01\n\x0eSysSetDateTime\x12\x0c\n\x04Year\x18\x01 \x01(\x05\x12\r\n\x05Month\x18\x02 \x01(\x05\x12\x0c\n\x04\x44\x61te\x18\x03 \x01(\x05\x12\x0c\n\x04Week\x18\x04 \x01(\x05\x12\r\n\x05Hours\x18\x05 \x01(\x05\x12\x0f\n\x07Minutes\x18\x06 \x01(\x05\x12\x0f\n\x07Seconds\x18\x07 \x01(\x05\x12\x10\n\x08timeZone\x18\x08 \x01(\x05\x12\x10\n\x08\x64\x61ylight\x18\t \x01(\x05\"V\n\nSysJobPlan\x12\r\n\x05jobId\x18\x01 \x01(\x03\x12\x0f\n\x07jobMode\x18\x02 \x01(\x05\x12\x13\n\x0brainTactics\x18\x03 \x01(\x05\x12\x13\n\x0bknifeHeight\x18\x04 \x01(\x05\"\"\n\rSysDevErrCode\x12\x11\n\terrorCode\x18\x01 \x01(\x05\"!\n\x0cSysBoardType\x12\x11\n\tboardType\x18\x01 \x01(\x05\"5\n\x0cSysSwVersion\x12\x11\n\tboardType\x18\x01 \x01(\x05\x12\x12\n\nversionLen\x18\x02 \x01(\x05\"1\n\rSysDelJobPlan\x12\x10\n\x08\x64\x65viceId\x18\x01 \x01(\t\x12\x0e\n\x06planId\x18\x02 \x01(\t\"\xec\x01\n\x0eSysJobPlanTime\x12\x0e\n\x06planId\x18\x01 \x01(\x03\x12\x16\n\x0estart_job_time\x18\x02 \x01(\x05\x12\x14\n\x0c\x65nd_job_time\x18\x03 \x01(\x05\x12\x13\n\x0btime_in_day\x18\x04 \x01(\x05\x12\x15\n\rjob_plan_mode\x18\x05 \x01(\x05\x12\x17\n\x0fjob_plan_enable\x18\x06 \x01(\x05\x12\x0f\n\x07weekDay\x18\x07 \x03(\x05\x12\x15\n\rtimeInWeekDay\x18\x08 \x03(\x05\x12\x10\n\x08\x65veryDay\x18\t \x01(\x05\x12\x1d\n\x08job_plan\x18\n \x01(\x0b\x32\x0b.SysJobPlan\"k\n\nSysMowInfo\x12\x13\n\x0b\x64\x65viceState\x18\x01 \x01(\x05\x12\x0e\n\x06\x62\x61tVal\x18\x02 \x01(\x05\x12\x13\n\x0bknifeHeight\x18\x03 \x01(\x05\x12\x11\n\tRTKstatus\x18\x04 \x01(\x05\x12\x10\n\x08RTKstars\x18\x05 \x01(\x05\";\n\x0eSysOptiLineAck\x12\x13\n\x0bresponesCmd\x18\x01 \x01(\x05\x12\x14\n\x0c\x63urrentFrame\x18\x02 \x01(\x05\"5\n\nSysCommCmd\x12\n\n\x02rw\x18\x01 \x01(\x05\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontext\x18\x03 \x01(\x05\"H\n\x15SysUploadFileProgress\x12\r\n\x05\x62izId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x05\x12\x10\n\x08progress\x18\x03 \x01(\x05\"\x1f\n\x0cSysErrorCode\x12\x0f\n\x07\x63ode_no\x18\x01 \x01(\x05\"\x1e\n\tSysBorder\x12\x11\n\tborderval\x18\x01 \x01(\x05\"*\n\x10SysPlanJobStatus\x12\x16\n\x0eplanjob_status\x18\x01 \x01(\x05\"=\n\x0fSysKnifeControl\x12\x14\n\x0cknife_status\x18\x01 \x01(\x05\x12\x14\n\x0cknife_height\x18\x02 \x01(\x05\"+\n\x14SysResetSystemStatus\x12\x13\n\x0breset_staus\x18\x01 \x01(\x05\"C\n\x1bSysResetBladeUsedTimeStatus\x12$\n\x1creset_blade_used_time_status\x18\x01 \x01(\x05\"\x8a\x01\n\rTimeCtrlLight\x12\x0f\n\x07operate\x18\x01 \x01(\x05\x12\x0e\n\x06\x65nable\x18\x02 \x01(\x05\x12\x12\n\nstart_hour\x18\x03 \x01(\x05\x12\x11\n\tstart_min\x18\x04 \x01(\x05\x12\x10\n\x08\x65nd_hour\x18\x05 \x01(\x05\x12\x0f\n\x07\x65nd_min\x18\x06 \x01(\x05\x12\x0e\n\x06\x61\x63tion\x18\x07 \x01(\x05\"3\n\x10vision_point_msg\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"\\\n\x15vision_point_info_msg\x12\r\n\x05label\x18\x01 \x01(\x05\x12\x0b\n\x03num\x18\x02 \x01(\x05\x12\'\n\x0cvision_point\x18\x03 \x03(\x0b\x32\x11.vision_point_msg\"\x9a\x01\n\x13vio_to_app_info_msg\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x12\x0f\n\x07heading\x18\x03 \x01(\x01\x12\x11\n\tvio_state\x18\x04 \x01(\x05\x12\x12\n\nbrightness\x18\x05 \x01(\x05\x12\x1a\n\x12\x64\x65tect_feature_num\x18\x06 \x01(\x05\x12\x19\n\x11track_feature_num\x18\x07 \x01(\x05\"1\n\x14vision_statistic_msg\x12\x0c\n\x04mean\x18\x01 \x01(\x02\x12\x0b\n\x03var\x18\x02 \x01(\x02\"m\n\x19vision_statistic_info_msg\x12\x11\n\ttimestamp\x18\x01 \x01(\x01\x12\x0b\n\x03num\x18\x02 \x01(\x05\x12\x30\n\x11vision_statistics\x18\x03 \x03(\x0b\x32\x15.vision_statistic_msg\"\xd3\x01\n\x1asystemRapidStateTunnel_msg\x12\x18\n\x10rapid_state_data\x18\x01 \x03(\x03\x12\x31\n\x11vision_point_info\x18\x02 \x03(\x0b\x32\x16.vision_point_info_msg\x12-\n\x0fvio_to_app_info\x18\x03 \x01(\x0b\x32\x14.vio_to_app_info_msg\x12\x39\n\x15vision_statistic_info\x18\x04 \x01(\x0b\x32\x1a.vision_statistic_info_msg\"4\n\x19systemTardStateTunnel_msg\x12\x17\n\x0ftard_state_data\x18\x01 \x03(\x03\".\n\x13systemUpdateBuf_msg\x12\x17\n\x0fupdate_buf_data\x18\x01 \x03(\x03\"\x9e\x01\n\x0fSysOffChipFlash\x12\x16\n\x02op\x18\x01 \x01(\x0e\x32\n.Operation\x12\x16\n\x02id\x18\x02 \x01(\x0e\x32\n.OffPartId\x12\x12\n\nstart_addr\x18\x03 \x01(\r\x12\x0e\n\x06offset\x18\x04 \x01(\r\x12\x0e\n\x06length\x18\x05 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\x0c\x12\x0c\n\x04\x63ode\x18\x07 \x01(\x05\x12\x0b\n\x03msg\x18\x08 \x01(\t\"-\n\x14systemTmpCycleTx_msg\x12\x15\n\rcycle_tx_data\x18\x01 \x03(\x03\"%\n\nLoraCfgReq\x12\n\n\x02op\x18\x01 \x01(\x05\x12\x0b\n\x03\x63\x66g\x18\x02 \x01(\t\"F\n\nLoraCfgRsp\x12\x0e\n\x06result\x18\x01 \x01(\x05\x12\n\n\x02op\x18\x02 \x01(\x05\x12\x0b\n\x03\x63\x66g\x18\x03 \x01(\t\x12\x0f\n\x07\x66\x61\x63_cfg\x18\x04 \x01(\x0c\">\n\x0bmod_fw_info\x12\x0c\n\x04type\x18\x01 \x01(\x05\x12\x10\n\x08identify\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\"L\n\x0e\x64\x65vice_fw_info\x12\x0e\n\x06result\x18\x01 \x01(\x05\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x19\n\x03mod\x18\x03 \x03(\x0b\x32\x0c.mod_fw_info\"@\n\x11mow_to_app_info_t\x12\x0c\n\x04type\x18\x01 \x01(\x05\x12\x0b\n\x03\x63md\x18\x02 \x01(\x05\x12\x10\n\x08mow_data\x18\x03 \x03(\x05\"a\n\x1a\x64\x65vice_product_type_info_t\x12\x0e\n\x06result\x18\x01 \x01(\x05\x12\x19\n\x11main_product_type\x18\x02 \x01(\t\x12\x18\n\x10sub_product_type\x18\x03 \x01(\t\"P\n\x0fQCAppTestExcept\x12\x13\n\x0b\x65xcept_type\x18\x01 \x01(\t\x12(\n\nconditions\x18\x02 \x03(\x0b\x32\x14.QCAppTestConditions\"t\n\x13QCAppTestConditions\x12\x11\n\tcond_type\x18\x01 \x01(\t\x12\x0f\n\x07int_val\x18\x02 \x01(\x05\x12\x11\n\tfloat_val\x18\x03 \x01(\x02\x12\x12\n\ndouble_val\x18\x04 \x01(\x01\x12\x12\n\nstring_val\x18\x05 \x01(\t\"\x99\x01\n\x19mow_to_app_qctools_info_t\x12\x1a\n\x04type\x18\x01 \x01(\x0e\x32\x0c.QCAppTestId\x12\x16\n\x0etimeOfDuration\x18\x02 \x01(\x05\x12\x0e\n\x06result\x18\x03 \x01(\x05\x12\x16\n\x0eresult_details\x18\x04 \x01(\t\x12 \n\x06\x65xcept\x18\x05 \x03(\x0b\x32\x10.QCAppTestExcept\"O\n\x16mCtrlSimulationCmdData\x12\x0e\n\x06subCmd\x18\x01 \x01(\x05\x12\x10\n\x08param_id\x18\x02 \x01(\x05\x12\x13\n\x0bparam_value\x18\x03 \x03(\x05\"7\n\x1c\x61pp_to_dev_get_mqtt_config_t\x12\x17\n\x0fget_mqtt_config\x18\x01 \x01(\x05\"H\n\x1c\x64\x65v_to_app_get_mqtt_config_t\x12\x12\n\nrtk_status\x18\x01 \x01(\x05\x12\x14\n\x0crtk_base_num\x18\x02 \x01(\t\"t\n\x19\x61pp_to_dev_set_mqtt_rtk_t\x12$\n\x0cset_rtk_mode\x18\x01 \x01(\x0e\x32\x0e.rtk_used_type\x12\x16\n\x0estop_nrtk_flag\x18\x02 \x01(\x05\x12\x19\n\x11set_nrtk_net_mode\x18\x03 \x01(\x05\"7\n\x19\x64\x65v_to_app_set_mqtt_rtk_t\x12\x1a\n\x12set_rtk_mode_error\x18\x01 \x01(\x05\"\x8f\x01\n\x08rpt_lora\x12\x16\n\x0epair_code_scan\x18\x01 \x01(\x05\x12\x19\n\x11pair_code_channel\x18\x02 \x01(\x05\x12\x17\n\x0fpair_code_locid\x18\x03 \x01(\x05\x12\x17\n\x0fpair_code_netid\x18\x04 \x01(\x05\x12\x1e\n\x16lora_connection_status\x18\x05 \x01(\x05\"\xd4\x01\n\x10mqtt_rtk_connect\x12\"\n\nrtk_switch\x18\x01 \x01(\x0e\x32\x0e.rtk_used_type\x12\x13\n\x0brtk_channel\x18\x02 \x01(\x05\x12\x14\n\x0crtk_base_num\x18\x03 \x01(\t\x12\x10\n\x08latitude\x18\x04 \x01(\x01\x12\x11\n\tlongitude\x18\x05 \x01(\x01\x12\x1f\n\x17nrtk_map_convert_status\x18\x06 \x01(\x05\x12\x15\n\rnrtk_net_mode\x18\x07 \x01(\x05\x12\x14\n\x0cnew_rtk_mode\x18\x08 \x01(\x05\"\x86\x01\n\tpos_score\x12\x13\n\x0brover_score\x18\x01 \x01(\r\x12\x13\n\x0brover_level\x18\x02 \x01(\r\x12\x12\n\nbase_score\x18\x03 \x01(\r\x12\x12\n\nbase_level\x18\x04 \x01(\r\x12\x12\n\nbase_moved\x18\x05 \x01(\r\x12\x13\n\x0b\x62\x61se_moving\x18\x06 \x01(\r\"\xbb\x02\n\x07rpt_rtk\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x11\n\tpos_level\x18\x02 \x01(\x05\x12\x11\n\tgps_stars\x18\x03 \x01(\x05\x12\x0b\n\x03\x61ge\x18\x04 \x01(\x05\x12\x0f\n\x07lat_std\x18\x05 \x01(\x05\x12\x0f\n\x07lon_std\x18\x06 \x01(\x05\x12\x10\n\x08l2_stars\x18\x07 \x01(\x05\x12\x12\n\ndis_status\x18\x08 \x01(\x03\x12\x17\n\x0ftop4_total_mean\x18\t \x01(\x03\x12\x15\n\rco_view_stars\x18\n \x01(\x05\x12\r\n\x05reset\x18\x0b \x01(\x05\x12\x1c\n\tlora_info\x18\x0c \x01(\x0b\x32\t.rpt_lora\x12(\n\rmqtt_rtk_info\x18\r \x01(\x0b\x32\x11.mqtt_rtk_connect\x12\x1e\n\nscore_info\x18\x0e \x01(\x0b\x32\n.pos_score\"\x86\x01\n\x10rpt_dev_location\x12\x12\n\nreal_pos_x\x18\x01 \x01(\x05\x12\x12\n\nreal_pos_y\x18\x02 \x01(\x05\x12\x13\n\x0breal_toward\x18\x03 \x01(\x05\x12\x10\n\x08pos_type\x18\x04 \x01(\x05\x12\x11\n\tzone_hash\x18\x05 \x01(\x03\x12\x10\n\x08\x62ol_hash\x18\x06 \x01(\x03\"4\n\x13vio_survival_info_t\x12\x1d\n\x15vio_survival_distance\x18\x01 \x01(\x02\";\n\x12\x63ollector_status_t\x12%\n\x1d\x63ollector_installation_status\x18\x01 \x01(\x05\"\"\n\x0clock_state_t\x12\x12\n\nlock_state\x18\x01 \x01(\r\"\x8b\x03\n\x0erpt_dev_status\x12\x12\n\nsys_status\x18\x01 \x01(\x05\x12\x14\n\x0c\x63harge_state\x18\x02 \x01(\x05\x12\x13\n\x0b\x62\x61ttery_val\x18\x03 \x01(\x05\x12\x15\n\rsensor_status\x18\x04 \x01(\x05\x12\x13\n\x0blast_status\x18\x05 \x01(\x05\x12\x16\n\x0esys_time_stamp\x18\x06 \x01(\x03\x12\x14\n\x0cvslam_status\x18\x07 \x01(\x05\x12\x1c\n\tmnet_info\x18\x08 \x01(\x0b\x32\t.MnetInfo\x12/\n\x11vio_survival_info\x18\t \x01(\x0b\x32\x14.vio_survival_info_t\x12-\n\x10\x63ollector_status\x18\n \x01(\x0b\x32\x13.collector_status_t\x12!\n\nlock_state\x18\x0b \x01(\x0b\x32\r.lock_state_t\x12\x19\n\x11self_check_status\x18\x0c \x01(\r\x12$\n\x08\x66pv_info\x18\r \x01(\x0b\x32\x12.fpv_to_app_info_t\"-\n\tnet_speed\x12\x10\n\x08\x64ownload\x18\x01 \x01(\r\x12\x0e\n\x06upload\x18\x02 \x01(\r\"\xce\x02\n\x12rpt_connect_status\x12\x14\n\x0c\x63onnect_type\x18\x01 \x01(\x05\x12\x10\n\x08\x62le_rssi\x18\x02 \x01(\x05\x12\x11\n\twifi_rssi\x18\x03 \x01(\x05\x12\x11\n\tlink_type\x18\x04 \x01(\x05\x12\x11\n\tmnet_rssi\x18\x05 \x01(\x05\x12\x11\n\tmnet_inet\x18\x06 \x01(\x05\x12 \n\x08used_net\x18\x07 \x01(\x0e\x32\x0e.net_used_type\x12\x1a\n\x08mnet_cfg\x18\x08 \x01(\x0b\x32\x08.MnetCfg\x12!\n\rdev_net_speed\x18\t \x01(\x0b\x32\n.net_speed\x12\x17\n\x0fiot_wifi_report\x18\n \x01(\x08\x12\x16\n\x0eiot_con_status\x18\x0b \x01(\x05\x12\x17\n\x0fwifi_con_status\x18\x0c \x01(\x05\x12\x19\n\x11wifi_is_available\x18\r \x01(\x05\",\n\x13nav_heading_state_t\x12\x15\n\rheading_state\x18\x01 \x01(\r\"\xfe\x03\n\x08rpt_work\x12\x0c\n\x04plan\x18\x01 \x01(\x05\x12\x11\n\tpath_hash\x18\x02 \x01(\x03\x12\x10\n\x08progress\x18\x03 \x01(\x05\x12\x0c\n\x04\x61rea\x18\x04 \x01(\x05\x12\x0f\n\x07\x62p_info\x18\x05 \x01(\x05\x12\x0f\n\x07\x62p_hash\x18\x06 \x01(\x03\x12\x10\n\x08\x62p_pos_x\x18\x07 \x01(\x05\x12\x10\n\x08\x62p_pos_y\x18\x08 \x01(\x05\x12\x15\n\rreal_path_num\x18\t \x01(\x03\x12\x12\n\npath_pos_x\x18\n \x01(\x05\x12\x12\n\npath_pos_y\x18\x0b \x01(\x05\x12\x14\n\x0cub_zone_hash\x18\x0c \x01(\x03\x12\x14\n\x0cub_path_hash\x18\r \x01(\x03\x12\x15\n\rinit_cfg_hash\x18\x0e \x01(\x03\x12\x15\n\rub_ecode_hash\x18\x0f \x01(\x03\x12\x14\n\x0cnav_run_mode\x18\x10 \x01(\x05\x12\x18\n\x10test_mode_status\x18\x11 \x01(\x03\x12\x15\n\rman_run_speed\x18\x12 \x01(\x05\x12\x17\n\x0fnav_edit_status\x18\x13 \x01(\x05\x12\x14\n\x0cknife_height\x18\x14 \x01(\x05\x12/\n\x11nav_heading_state\x18\x15 \x01(\x0b\x32\x14.nav_heading_state_t\x12\x15\n\rcutter_offset\x18\x16 \x01(\x02\x12\x14\n\x0c\x63utter_width\x18\x17 \x01(\x02\"C\n\nblade_used\x12\x17\n\x0f\x62lade_used_time\x18\x01 \x01(\x05\x12\x1c\n\x14\x62lade_used_warn_time\x18\x02 \x01(\x05\"=\n\x1duser_set_blade_used_warn_time\x12\x1c\n\x14\x62lade_used_warn_time\x18\x01 \x01(\x05\"l\n\x0crpt_maintain\x12\x0f\n\x07mileage\x18\x01 \x01(\x03\x12\x11\n\twork_time\x18\x02 \x01(\x05\x12\x12\n\nbat_cycles\x18\x03 \x01(\x05\x12$\n\x0f\x62lade_used_time\x18\x04 \x01(\x0b\x32\x0b.blade_used\"[\n\x11\x66pv_to_app_info_t\x12\x10\n\x08\x66pv_flag\x18\x01 \x01(\x05\x12\x16\n\x0ewifi_available\x18\x02 \x01(\x05\x12\x1c\n\x14mobile_net_available\x18\x03 \x01(\x05\"\xa4\x01\n\x14rpt_basestation_info\x12\x11\n\tver_major\x18\x01 \x01(\r\x12\x11\n\tver_minor\x18\x02 \x01(\r\x12\x11\n\tver_patch\x18\x03 \x01(\r\x12\x11\n\tver_build\x18\x04 \x01(\r\x12\x1a\n\x12\x62\x61sestation_status\x18\x05 \x01(\r\x12$\n\x1c\x63onnect_status_since_poweron\x18\x06 \x01(\r\"\x8f\x01\n\x0freport_info_cfg\x12\x15\n\x03\x61\x63t\x18\x01 \x01(\x0e\x32\x08.rpt_act\x12\x0f\n\x07timeout\x18\x02 \x01(\x05\x12\x0e\n\x06period\x18\x03 \x01(\x05\x12\x18\n\x10no_change_period\x18\x04 \x01(\x05\x12\r\n\x05\x63ount\x18\x05 \x01(\x05\x12\x1b\n\x03sub\x18\x06 \x03(\x0e\x32\x0e.rpt_info_type\"\xed\x03\n\x10report_info_data\x12$\n\x07\x63onnect\x18\x01 \x01(\x0b\x32\x13.rpt_connect_status\x12\x1c\n\x03\x64\x65v\x18\x02 \x01(\x0b\x32\x0f.rpt_dev_status\x12\x15\n\x03rtk\x18\x03 \x01(\x0b\x32\x08.rpt_rtk\x12$\n\tlocations\x18\x04 \x03(\x0b\x32\x11.rpt_dev_location\x12\x17\n\x04work\x18\x05 \x01(\x0b\x32\t.rpt_work\x12 \n\x07\x66w_info\x18\x06 \x01(\x0b\x32\x0f.device_fw_info\x12\x1f\n\x08maintain\x18\x07 \x01(\x0b\x32\r.rpt_maintain\x12\x31\n\x11vision_point_info\x18\x08 \x03(\x0b\x32\x16.vision_point_info_msg\x12-\n\x0fvio_to_app_info\x18\t \x01(\x0b\x32\x14.vio_to_app_info_msg\x12\x39\n\x15vision_statistic_info\x18\n \x01(\x0b\x32\x1a.vision_statistic_info_msg\x12/\n\x10\x62\x61sestation_info\x18\x0b \x01(\x0b\x32\x15.rpt_basestation_info\x12.\n\x15\x63utter_work_mode_info\x18\x0c \x01(\x0b\x32\x0f.rpt_cutter_rpm\"U\n\x15\x64\x65\x62ug_common_report_t\x12\x0e\n\x06m_name\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x12\x10\n\x08gen_time\x18\x04 \x01(\x03\"W\n\x17\x64\x65\x62ug_errocode_report_t\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\r\n\x05mname\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x12\x10\n\x08gen_time\x18\x04 \x01(\x03\" \n\x0e\x64\x65\x62ug_enable_t\x12\x0e\n\x06\x65nbale\x18\x01 \x01(\x05\"^\n\x17\x64\x65\x62ug_res_cfg_ability_t\x12\x12\n\ntotal_keys\x18\x01 \x01(\x05\x12\x12\n\ncur_key_id\x18\x02 \x01(\x05\x12\x0c\n\x04keys\x18\x03 \x01(\t\x12\r\n\x05value\x18\x04 \x01(\t\".\n\x10\x64\x65\x62ug_cfg_read_t\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"/\n\x11\x64\x65\x62ug_cfg_write_t\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xa7\x01\n\nmsgbus_pkt\x12\x0c\n\x04type\x18\x01 \x01(\x05\x12\x13\n\x0btypeCommand\x18\x02 \x01(\x05\x12\x14\n\x0crecvDeviceId\x18\x03 \x01(\x05\x12\x14\n\x0csendDeviceId\x18\x04 \x01(\x05\x12\x12\n\ndataLength\x18\x05 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\t\x12\x0c\n\x04\x63trl\x18\x07 \x01(\x05\x12\x0c\n\x04\x66lag\x18\x08 \x01(\x05\x12\x0c\n\x04seqs\x18\t \x01(\x05\"\xb1\x01\n\x13response_set_mode_t\x12\x0e\n\x06statue\x18\x01 \x01(\x05\x12\x15\n\rset_work_mode\x18\x02 \x01(\x05\x12\x15\n\rcur_work_mode\x18\x03 \x01(\x05\x12\x17\n\x0fstart_work_time\x18\x04 \x01(\x03\x12\x15\n\rend_work_time\x18\x05 \x01(\x03\x12\x15\n\rinterruptflag\x18\x06 \x01(\x05\x12\x15\n\rcur_work_time\x18\x07 \x01(\x05\"I\n\x0erpt_cutter_rpm\x12\x1b\n\x13\x63urrent_cutter_mode\x18\x01 \x01(\x05\x12\x1a\n\x12\x63urrent_cutter_rpm\x18\x02 \x01(\x05\"\x8d\x02\n\x0c\x64\x65v_statue_t\x12\x12\n\nsys_status\x18\x01 \x01(\x05\x12\x15\n\rcharge_status\x18\x02 \x01(\x05\x12\x0f\n\x07\x62\x61t_val\x18\x03 \x01(\x05\x12\x14\n\x0cwheel_status\x18\x04 \x01(\x05\x12\x13\n\x0bpump_status\x18\x05 \x01(\x05\x12\x11\n\twork_mode\x18\x06 \x01(\x05\x12\r\n\x05model\x18\x07 \x01(\x05\x12\x10\n\x08\x62le_rssi\x18\x08 \x01(\x05\x12\x11\n\twifi_rssi\x18\t \x01(\x05\x12\x1b\n\x13wifi_connect_status\x18\n \x01(\x05\x12\x1a\n\x12iot_connect_status\x18\x0b \x01(\x05\x12\x16\n\x0ewifi_available\x18\x0c \x01(\x05\"2\n\rreport_info_t\x12!\n\ndev_status\x18\x01 \x01(\x0b\x32\r.dev_statue_t\" \n\x0bwork_mode_t\x12\x11\n\twork_mode\x18\x01 \x01(\x05\"Q\n\x0especial_mode_t\x12\x12\n\nstair_mode\x18\x01 \x01(\x05\x12\x14\n\x0cviolent_mode\x18\x02 \x01(\x05\x12\x15\n\rberthing_mode\x18\x03 \x01(\x05\"(\n\x11set_peripherals_t\x12\x13\n\x0b\x62uzz_enable\x18\x01 \x01(\x05\"K\n\x10\x64\x65\x62ug_sun_time_t\x12\x0e\n\x06subCmd\x18\x01 \x01(\x05\x12\x13\n\x0bsunRiseTime\x18\x02 \x01(\x05\x12\x12\n\nsunSetTime\x18\x03 \x01(\x05\"l\n\x12remote_reset_req_t\x12\r\n\x05magic\x18\x01 \x01(\x05\x12\r\n\x05\x62izid\x18\x02 \x01(\x03\x12\x12\n\nreset_mode\x18\x03 \x01(\x05\x12\x13\n\x0b\x66orce_reset\x18\x04 \x01(\x05\x12\x0f\n\x07\x61\x63\x63ount\x18\x05 \x01(\x03\"S\n\x12remote_reset_rsp_t\x12\r\n\x05magic\x18\x01 \x01(\x05\x12\r\n\x05\x62izid\x18\x02 \x01(\x03\x12\x1f\n\x06result\x18\x03 \x01(\x0e\x32\x0f.Command_Result\"\xb1\x16\n\x07MctlSys\x12\"\n\rtoapp_batinfo\x18\x01 \x01(\x0b\x32\t.SysBatUpH\x00\x12)\n\x10toapp_work_state\x18\x02 \x01(\x0b\x32\r.SysWorkStateH\x00\x12*\n\x0ftodev_time_zone\x18\x03 \x01(\x0b\x32\x0f.SysSetTimeZoneH\x00\x12*\n\x0ftodev_data_time\x18\x04 \x01(\x0b\x32\x0f.SysSetDateTimeH\x00\x12\x1f\n\x08job_plan\x18\x06 \x01(\x0b\x32\x0b.SysJobPlanH\x00\x12(\n\x0etoapp_err_code\x18\x07 \x01(\x0b\x32\x0e.SysDevErrCodeH\x00\x12.\n\x13todev_job_plan_time\x18\n \x01(\x0b\x32\x0f.SysJobPlanTimeH\x00\x12%\n\x0etoapp_mow_info\x18\x0b \x01(\x0b\x32\x0b.SysMowInfoH\x00\x12&\n\x0f\x62idire_comm_cmd\x18\x0c \x01(\x0b\x32\x0b.SysCommCmdH\x00\x12\x16\n\x0cplan_job_del\x18\x0e \x01(\x03H\x00\x12\x1c\n\x06\x62order\x18\x0f \x01(\x0b\x32\n.SysBorderH\x00\x12.\n\x11toapp_plan_status\x18\x12 \x01(\x0b\x32\x11.SysPlanJobStatusH\x00\x12\x34\n\x12toapp_ul_fprogress\x18\x13 \x01(\x0b\x32\x16.SysUploadFileProgressH\x00\x12*\n\x10todev_deljobplan\x18\x14 \x01(\x0b\x32\x0e.SysDelJobPlanH\x00\x12\x1b\n\x11todev_mow_info_up\x18\x15 \x01(\x05H\x00\x12,\n\x10todev_knife_ctrl\x18\x16 \x01(\x0b\x32\x10.SysKnifeControlH\x00\x12\x1c\n\x12todev_reset_system\x18\x17 \x01(\x05H\x00\x12:\n\x19todev_reset_system_status\x18\x18 \x01(\x0b\x32\x15.SysResetSystemStatusH\x00\x12=\n\x16systemRapidStateTunnel\x18\x19 \x01(\x0b\x32\x1b.systemRapidStateTunnel_msgH\x00\x12;\n\x15systemTardStateTunnel\x18\x1a \x01(\x0b\x32\x1a.systemTardStateTunnel_msgH\x00\x12/\n\x0fsystemUpdateBuf\x18\x1b \x01(\x0b\x32\x14.systemUpdateBuf_msgH\x00\x12/\n\x15todev_time_ctrl_light\x18\x1c \x01(\x0b\x32\x0e.TimeCtrlLightH\x00\x12\x31\n\x10systemTmpCycleTx\x18\x1d \x01(\x0b\x32\x15.systemTmpCycleTx_msgH\x00\x12\x30\n\x14todev_off_chip_flash\x18\x1e \x01(\x0b\x32\x10.SysOffChipFlashH\x00\x12\x1f\n\x15todev_get_dev_fw_info\x18\x1f \x01(\x05H\x00\x12,\n\x11toapp_dev_fw_info\x18  \x01(\x0b\x32\x0f.device_fw_infoH\x00\x12)\n\x12todev_lora_cfg_req\x18! \x01(\x0b\x32\x0b.LoraCfgReqH\x00\x12)\n\x12toapp_lora_cfg_rsp\x18\" \x01(\x0b\x32\x0b.LoraCfgRspH\x00\x12-\n\x0fmow_to_app_info\x18# \x01(\x0b\x32\x12.mow_to_app_info_tH\x00\x12?\n\x18\x64\x65vice_product_type_info\x18$ \x01(\x0b\x32\x1b.device_product_type_info_tH\x00\x12=\n\x17mow_to_app_qctools_info\x18% \x01(\x0b\x32\x1a.mow_to_app_qctools_info_tH\x00\x12,\n\x10todev_report_cfg\x18& \x01(\x0b\x32\x10.report_info_cfgH\x00\x12.\n\x11toapp_report_data\x18\' \x01(\x0b\x32\x11.report_info_dataH\x00\x12\x31\n\x0esimulation_cmd\x18* \x01(\x0b\x32\x17.mCtrlSimulationCmdDataH\x00\x12G\n\x1e\x61pp_to_dev_get_mqtt_config_msg\x18+ \x01(\x0b\x32\x1d.app_to_dev_get_mqtt_config_tH\x00\x12G\n\x1e\x64\x65v_to_app_get_mqtt_config_msg\x18, \x01(\x0b\x32\x1d.dev_to_app_get_mqtt_config_tH\x00\x12\x41\n\x1b\x61pp_to_dev_set_mqtt_rtk_msg\x18- \x01(\x0b\x32\x1a.app_to_dev_set_mqtt_rtk_tH\x00\x12\x41\n\x1b\x64\x65v_to_app_set_mqtt_rtk_msg\x18. \x01(\x0b\x32\x1a.dev_to_app_set_mqtt_rtk_tH\x00\x12%\n\x1btodev_reset_blade_used_time\x18/ \x01(\x05H\x00\x12J\n\"todev_reset_blade_used_time_status\x18\x30 \x01(\x0b\x32\x1c.SysResetBladeUsedTimeStatusH\x00\x12#\n\x19todev_factor_reset_system\x18\x31 \x01(\x05H\x00\x12>\n\x14\x62lade_used_warn_time\x18\x32 \x01(\x0b\x32\x1e.user_set_blade_used_warn_timeH\x00\x12\x35\n\x13\x64\x65\x62ug_common_report\x18\x33 \x01(\x0b\x32\x16.debug_common_report_tH\x00\x12\x39\n\x15\x64\x65\x62ug_errocode_report\x18\x34 \x01(\x0b\x32\x18.debug_errocode_report_tH\x00\x12\'\n\x0c\x64\x65\x62ug_enable\x18\x35 \x01(\x0b\x32\x0f.debug_enable_tH\x00\x12+\n\x0e\x64\x65\x62ug_cfg_read\x18\x36 \x01(\x0b\x32\x11.debug_cfg_read_tH\x00\x12-\n\x0f\x64\x65\x62ug_cfg_write\x18\x37 \x01(\x0b\x32\x12.debug_cfg_write_tH\x00\x12\x39\n\x15\x64\x65\x62ug_res_cfg_ability\x18\x38 \x01(\x0b\x32\x18.debug_res_cfg_ability_tH\x00\x12$\n\rto_dev_msgbus\x18\x39 \x01(\x0b\x32\x0b.msgbus_pktH\x00\x12$\n\rto_app_msgbus\x18: \x01(\x0b\x32\x0b.msgbus_pktH\x00\x12\x31\n\x11response_set_mode\x18; \x01(\x0b\x32\x14.response_set_mode_tH\x00\x12%\n\x0breport_info\x18< \x01(\x0b\x32\x0e.report_info_tH\x00\x12%\n\rset_work_mode\x18= \x01(\x0b\x32\x0c.work_mode_tH\x00\x12+\n\x10set_special_mode\x18> \x01(\x0b\x32\x0f.special_mode_tH\x00\x12-\n\x0fset_peripherals\x18? \x01(\x0b\x32\x12.set_peripherals_tH\x00\x12\x30\n\x13to_dev_set_sun_time\x18@ \x01(\x0b\x32\x11.debug_sun_time_tH\x00\x12\x32\n\x13to_dev_remote_reset\x18\x41 \x01(\x0b\x32\x13.remote_reset_req_tH\x00\x12\x32\n\x13to_app_remote_reset\x18\x42 \x01(\x0b\x32\x13.remote_reset_rsp_tH\x00\x12.\n\x13\x63urrent_cutter_mode\x18\x43 \x01(\x0b\x32\x0f.rpt_cutter_rpmH\x00\x42\x0b\n\tSubSysMsg*+\n\tOperation\x12\t\n\x05WRITE\x10\x00\x12\x08\n\x04READ\x10\x01\x12\t\n\x05\x45RASE\x10\x02*\x8d\x02\n\tOffPartId\x12\x13\n\x0fOFF_PART_DL_IMG\x10\x00\x12\x19\n\x15OFF_PART_UPDINFO_BACK\x10\x01\x12\x14\n\x10OFF_PART_UPDINFO\x10\x02\x12\x13\n\x0fOFF_PART_NAKEDB\x10\x03\x12\x14\n\x10OFF_PART_FLASHDB\x10\x04\x12\x18\n\x14OFF_PART_UPD_APP_IMG\x10\x05\x12\x18\n\x14OFF_PART_UPD_BMS_IMG\x10\x06\x12\x18\n\x14OFF_PART_UPD_TMP_IMG\x10\x07\x12\x15\n\x11OFF_PART_DEV_INFO\x10\x08\x12\x18\n\x14OFF_PART_NAKEDB_BACK\x10\t\x12\x10\n\x0cOFF_PART_MAX\x10\n*\xe5\x08\n\x0bQCAppTestId\x12!\n\x1dQC_APP_ITEM_ON_CHARGESATSTION\x10\x00\x12\x1a\n\x16QC_APP_TEST_X3_SPEAKER\x10\x01\x12)\n%QC_APP_TEST_STATIC_OBSTACLE_DETECTION\x10\x02\x12\"\n\x1eQC_APP_TEST_CHARGESTATION_TEMP\x10\x03\x12\x13\n\x0fQC_APP_ITEM_KEY\x10\x04\x12 \n\x1cQC_APP_TEST_BUMPER_FRONTLEFT\x10\x05\x12!\n\x1dQC_APP_TEST_BUMPER_FRONTRIGHT\x10\x06\x12\x14\n\x10QC_APP_TEST_STOP\x10\x07\x12\x16\n\x12QC_APP_TEST_UNLOCK\x10\x08\x12\x14\n\x10QC_APP_TEST_BUZZ\x10\t\x12\x14\n\x10QC_APP_TEST_LIFT\x10\n\x12\x16\n\x12QC_APP_ITEM_SENEOR\x10\x0b\x12\x19\n\x15QC_APP_TEST_ROLL_LEFT\x10\x0c\x12\x1a\n\x16QC_APP_TEST_ROLL_RIGHT\x10\r\x12\x1d\n\x19QC_APP_TEST_ULTRA_UNCOVER\x10\x0e\x12\x1c\n\x18QC_APP_TEST_ULTRA0_COVER\x10\x0f\x12\x1c\n\x18QC_APP_TEST_ULTRA1_COVER\x10\x10\x12\x1c\n\x18QC_APP_TEST_ULTRA2_COVER\x10\x11\x12\x14\n\x10QC_APP_TEST_RAIN\x10\x12\x12\x12\n\x0eQC_APP_ITEM_SQ\x10\x13\x12\x18\n\x14QC_APP_TEST_BLE_RSSI\x10\x14\x12 \n\x1cQC_APP_TEST_SATELLITES_ROVER\x10\x15\x12)\n%QC_APP_TEST_SATELLITES_REF_STATION_L1\x10\x16\x12)\n%QC_APP_TEST_SATELLITES_REF_STATION_L2\x10\x17\x12&\n\"QC_APP_TEST_SATELLITES_COMMON_VIEW\x10\x18\x12\x19\n\x15QC_APP_TEST_CNO_ROVER\x10\x19\x12\x1f\n\x1bQC_APP_TEST_CNO_REF_STATION\x10\x1a\x12\'\n#QC_APP_TEST_REF_STATION_LINK_STATUS\x10\x1b\x12\x1e\n\x1aQC_APP_TEST_LOCATION_STATE\x10\x1c\x12\x1d\n\x19QC_APP_TEST_CHARGE_STATUS\x10\x1d\x12\x1e\n\x1aQC_APP_TEST_PPS_EXTI_COUNT\x10\x1e\x12\x18\n\x14QC_APP_TEST_SAFE_KEY\x10\x32\x12\x19\n\x15QC_APP_TEST_LORA_RSSI\x10<\x12\x1a\n\x16QC_APP_TEST_WIPER_TEST\x10P\x12\x1d\n\x19QC_APP_TEST_HEADLAMP_TEST\x10Q\x12\x1f\n\x1bQC_APP_TEST_COMPLETE_SIGNAL\x10\x63\x12\x13\n\x0fQC_APP_TEST_MAX\x10\x64*L\n\rrtk_used_type\x12\x11\n\rRTK_USED_LORA\x10\x00\x12\x15\n\x11RTK_USED_INTERNET\x10\x01\x12\x11\n\rRTK_USED_NRTK\x10\x02*W\n\rnet_used_type\x12\x16\n\x12NET_USED_TYPE_NONE\x10\x00\x12\x16\n\x12NET_USED_TYPE_WIFI\x10\x01\x12\x16\n\x12NET_USED_TYPE_MNET\x10\x02*\xee\x01\n\rrpt_info_type\x12\x0f\n\x0bRIT_CONNECT\x10\x00\x12\x0f\n\x0bRIT_DEV_STA\x10\x01\x12\x0b\n\x07RIT_RTK\x10\x02\x12\x11\n\rRIT_DEV_LOCAL\x10\x03\x12\x0c\n\x08RIT_WORK\x10\x04\x12\x0f\n\x0bRIT_FW_INFO\x10\x05\x12\x10\n\x0cRIT_MAINTAIN\x10\x06\x12\x14\n\x10RIT_VISION_POINT\x10\x07\x12\x0b\n\x07RIT_VIO\x10\x08\x12\x18\n\x14RIT_VISION_STATISTIC\x10\t\x12\x18\n\x14RIT_BASESTATION_INFO\x10\n\x12\x13\n\x0fRIT_CUTTER_INFO\x10\x0b*4\n\x07rpt_act\x12\r\n\tRPT_START\x10\x00\x12\x0c\n\x08RPT_STOP\x10\x01\x12\x0c\n\x08RPT_KEEP\x10\x02*R\n\x0e\x43ommand_Result\x12\t\n\x05RS_OK\x10\x00\x12\x11\n\rRS_FAIL_MAGIC\x10\x01\x12\x0f\n\x0bRS_FAIL_OTA\x10\x02\x12\x11\n\rRS_FAIL_SLOPE\x10\x03\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pymammotion.proto.mctrl_sys_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _OPERATION._serialized_start=11827
  _OPERATION._serialized_end=11870
  _OFFPARTID._serialized_start=11873
  _OFFPARTID._serialized_end=12142
  _QCAPPTESTID._serialized_start=12145
  _QCAPPTESTID._serialized_end=13270
  _RTK_USED_TYPE._serialized_start=13272
  _RTK_USED_TYPE._serialized_end=13348
  _NET_USED_TYPE._serialized_start=13350
  _NET_USED_TYPE._serialized_end=13437
  _RPT_INFO_TYPE._serialized_start=13440
  _RPT_INFO_TYPE._serialized_end=13678
  _RPT_ACT._serialized_start=13680
  _RPT_ACT._serialized_end=13732
  _COMMAND_RESULT._serialized_start=13734
  _COMMAND_RESULT._serialized_end=13816
  _SYSBATUP._serialized_start=70
  _SYSBATUP._serialized_end=96
  _SYSWORKSTATE._serialized_start=98
  _SYSWORKSTATE._serialized_end=188
  _SYSSETTIMEZONE._serialized_start=190
  _SYSSETTIMEZONE._serialized_end=243
  _SYSSETDATETIME._serialized_start=246
  _SYSSETDATETIME._serialized_end=404
  _SYSJOBPLAN._serialized_start=406
  _SYSJOBPLAN._serialized_end=492
  _SYSDEVERRCODE._serialized_start=494
  _SYSDEVERRCODE._serialized_end=528
  _SYSBOARDTYPE._serialized_start=530
  _SYSBOARDTYPE._serialized_end=563
  _SYSSWVERSION._serialized_start=565
  _SYSSWVERSION._serialized_end=618
  _SYSDELJOBPLAN._serialized_start=620
  _SYSDELJOBPLAN._serialized_end=669
  _SYSJOBPLANTIME._serialized_start=672
  _SYSJOBPLANTIME._serialized_end=908
  _SYSMOWINFO._serialized_start=910
  _SYSMOWINFO._serialized_end=1017
  _SYSOPTILINEACK._serialized_start=1019
  _SYSOPTILINEACK._serialized_end=1078
  _SYSCOMMCMD._serialized_start=1080
  _SYSCOMMCMD._serialized_end=1133
  _SYSUPLOADFILEPROGRESS._serialized_start=1135
  _SYSUPLOADFILEPROGRESS._serialized_end=1207
  _SYSERRORCODE._serialized_start=1209
  _SYSERRORCODE._serialized_end=1240
  _SYSBORDER._serialized_start=1242
  _SYSBORDER._serialized_end=1272
  _SYSPLANJOBSTATUS._serialized_start=1274
  _SYSPLANJOBSTATUS._serialized_end=1316
  _SYSKNIFECONTROL._serialized_start=1318
  _SYSKNIFECONTROL._serialized_end=1379
  _SYSRESETSYSTEMSTATUS._serialized_start=1381
  _SYSRESETSYSTEMSTATUS._serialized_end=1424
  _SYSRESETBLADEUSEDTIMESTATUS._serialized_start=1426
  _SYSRESETBLADEUSEDTIMESTATUS._serialized_end=1493
  _TIMECTRLLIGHT._serialized_start=1496
  _TIMECTRLLIGHT._serialized_end=1634
  _VISION_POINT_MSG._serialized_start=1636
  _VISION_POINT_MSG._serialized_end=1687
  _VISION_POINT_INFO_MSG._serialized_start=1689
  _VISION_POINT_INFO_MSG._serialized_end=1781
  _VIO_TO_APP_INFO_MSG._serialized_start=1784
  _VIO_TO_APP_INFO_MSG._serialized_end=1938
  _VISION_STATISTIC_MSG._serialized_start=1940
  _VISION_STATISTIC_MSG._serialized_end=1989
  _VISION_STATISTIC_INFO_MSG._serialized_start=1991
  _VISION_STATISTIC_INFO_MSG._serialized_end=2100
  _SYSTEMRAPIDSTATETUNNEL_MSG._serialized_start=2103
  _SYSTEMRAPIDSTATETUNNEL_MSG._serialized_end=2314
  _SYSTEMTARDSTATETUNNEL_MSG._serialized_start=2316
  _SYSTEMTARDSTATETUNNEL_MSG._serialized_end=2368
  _SYSTEMUPDATEBUF_MSG._serialized_start=2370
  _SYSTEMUPDATEBUF_MSG._serialized_end=2416
  _SYSOFFCHIPFLASH._serialized_start=2419
  _SYSOFFCHIPFLASH._serialized_end=2577
  _SYSTEMTMPCYCLETX_MSG._serialized_start=2579
  _SYSTEMTMPCYCLETX_MSG._serialized_end=2624
  _LORACFGREQ._serialized_start=2626
  _LORACFGREQ._serialized_end=2663
  _LORACFGRSP._serialized_start=2665
  _LORACFGRSP._serialized_end=2735
  _MOD_FW_INFO._serialized_start=2737
  _MOD_FW_INFO._serialized_end=2799
  _DEVICE_FW_INFO._serialized_start=2801
  _DEVICE_FW_INFO._serialized_end=2877
  _MOW_TO_APP_INFO_T._serialized_start=2879
  _MOW_TO_APP_INFO_T._serialized_end=2943
  _DEVICE_PRODUCT_TYPE_INFO_T._serialized_start=2945
  _DEVICE_PRODUCT_TYPE_INFO_T._serialized_end=3042
  _QCAPPTESTEXCEPT._serialized_start=3044
  _QCAPPTESTEXCEPT._serialized_end=3124
  _QCAPPTESTCONDITIONS._serialized_start=3126
  _QCAPPTESTCONDITIONS._serialized_end=3242
  _MOW_TO_APP_QCTOOLS_INFO_T._serialized_start=3245
  _MOW_TO_APP_QCTOOLS_INFO_T._serialized_end=3398
  _MCTRLSIMULATIONCMDDATA._serialized_start=3400
  _MCTRLSIMULATIONCMDDATA._serialized_end=3479
  _APP_TO_DEV_GET_MQTT_CONFIG_T._serialized_start=3481
  _APP_TO_DEV_GET_MQTT_CONFIG_T._serialized_end=3536
  _DEV_TO_APP_GET_MQTT_CONFIG_T._serialized_start=3538
  _DEV_TO_APP_GET_MQTT_CONFIG_T._serialized_end=3610
  _APP_TO_DEV_SET_MQTT_RTK_T._serialized_start=3612
  _APP_TO_DEV_SET_MQTT_RTK_T._serialized_end=3728
  _DEV_TO_APP_SET_MQTT_RTK_T._serialized_start=3730
  _DEV_TO_APP_SET_MQTT_RTK_T._serialized_end=3785
  _RPT_LORA._serialized_start=3788
  _RPT_LORA._serialized_end=3931
  _MQTT_RTK_CONNECT._serialized_start=3934
  _MQTT_RTK_CONNECT._serialized_end=4146
  _POS_SCORE._serialized_start=4149
  _POS_SCORE._serialized_end=4283
  _RPT_RTK._serialized_start=4286
  _RPT_RTK._serialized_end=4601
  _RPT_DEV_LOCATION._serialized_start=4604
  _RPT_DEV_LOCATION._serialized_end=4738
  _VIO_SURVIVAL_INFO_T._serialized_start=4740
  _VIO_SURVIVAL_INFO_T._serialized_end=4792
  _COLLECTOR_STATUS_T._serialized_start=4794
  _COLLECTOR_STATUS_T._serialized_end=4853
  _LOCK_STATE_T._serialized_start=4855
  _LOCK_STATE_T._serialized_end=4889
  _RPT_DEV_STATUS._serialized_start=4892
  _RPT_DEV_STATUS._serialized_end=5287
  _NET_SPEED._serialized_start=5289
  _NET_SPEED._serialized_end=5334
  _RPT_CONNECT_STATUS._serialized_start=5337
  _RPT_CONNECT_STATUS._serialized_end=5671
  _NAV_HEADING_STATE_T._serialized_start=5673
  _NAV_HEADING_STATE_T._serialized_end=5717
  _RPT_WORK._serialized_start=5720
  _RPT_WORK._serialized_end=6230
  _BLADE_USED._serialized_start=6232
  _BLADE_USED._serialized_end=6299
  _USER_SET_BLADE_USED_WARN_TIME._serialized_start=6301
  _USER_SET_BLADE_USED_WARN_TIME._serialized_end=6362
  _RPT_MAINTAIN._serialized_start=6364
  _RPT_MAINTAIN._serialized_end=6472
  _FPV_TO_APP_INFO_T._serialized_start=6474
  _FPV_TO_APP_INFO_T._serialized_end=6565
  _RPT_BASESTATION_INFO._serialized_start=6568
  _RPT_BASESTATION_INFO._serialized_end=6732
  _REPORT_INFO_CFG._serialized_start=6735
  _REPORT_INFO_CFG._serialized_end=6878
  _REPORT_INFO_DATA._serialized_start=6881
  _REPORT_INFO_DATA._serialized_end=7374
  _DEBUG_COMMON_REPORT_T._serialized_start=7376
  _DEBUG_COMMON_REPORT_T._serialized_end=7461
  _DEBUG_ERROCODE_REPORT_T._serialized_start=7463
  _DEBUG_ERROCODE_REPORT_T._serialized_end=7550
  _DEBUG_ENABLE_T._serialized_start=7552
  _DEBUG_ENABLE_T._serialized_end=7584
  _DEBUG_RES_CFG_ABILITY_T._serialized_start=7586
  _DEBUG_RES_CFG_ABILITY_T._serialized_end=7680
  _DEBUG_CFG_READ_T._serialized_start=7682
  _DEBUG_CFG_READ_T._serialized_end=7728
  _DEBUG_CFG_WRITE_T._serialized_start=7730
  _DEBUG_CFG_WRITE_T._serialized_end=7777
  _MSGBUS_PKT._serialized_start=7780
  _MSGBUS_PKT._serialized_end=7947
  _RESPONSE_SET_MODE_T._serialized_start=7950
  _RESPONSE_SET_MODE_T._serialized_end=8127
  _RPT_CUTTER_RPM._serialized_start=8129
  _RPT_CUTTER_RPM._serialized_end=8202
  _DEV_STATUE_T._serialized_start=8205
  _DEV_STATUE_T._serialized_end=8474
  _REPORT_INFO_T._serialized_start=8476
  _REPORT_INFO_T._serialized_end=8526
  _WORK_MODE_T._serialized_start=8528
  _WORK_MODE_T._serialized_end=8560
  _SPECIAL_MODE_T._serialized_start=8562
  _SPECIAL_MODE_T._serialized_end=8643
  _SET_PERIPHERALS_T._serialized_start=8645
  _SET_PERIPHERALS_T._serialized_end=8685
  _DEBUG_SUN_TIME_T._serialized_start=8687
  _DEBUG_SUN_TIME_T._serialized_end=8762
  _REMOTE_RESET_REQ_T._serialized_start=8764
  _REMOTE_RESET_REQ_T._serialized_end=8872
  _REMOTE_RESET_RSP_T._serialized_start=8874
  _REMOTE_RESET_RSP_T._serialized_end=8957
  _MCTLSYS._serialized_start=8960
  _MCTLSYS._serialized_end=11825
# @@protoc_insertion_point(module_scope)



"""App key and secret as taken from the mammotion android app."""

APP_KEY = "34231230"
APP_SECRET = "1ba85698bb10e19c6437413b61ba3445"
APP_VERSION = "1.11.130"
ALIYUN_DOMAIN = "api.link.aliyun.com"
MAMMOTION_DOMAIN = "https://id.mammotion.com"
MAMMOTION_API_DOMAIN = "https://domestic.mammotion.com"
MAMMOTION_CLIENT_ID = "MADKALUBAS"
MAMMOTION_CLIENT_SECRET = "GshzGRZJjuMUgd2sYHM7"

MAMMOTION_OUATH2_CLIENT_ID = "GxebgSt8si6pKqR"
MAMMOTION_OUATH2_CLIENT_SECRET = "JP0508SRJFa0A90ADpzLINDBxMa4Vj"



from base64 import b64decode
from dataclasses import dataclass
from typing import Annotated, Any, Literal

from google.protobuf import json_format
from mashumaro.config import BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin
from mashumaro.types import Alias, SerializableType

from pymammotion.proto import luba_msg_pb2


class Base64EncodedProtobuf(SerializableType):
    def __init__(self, proto: str) -> None:
        self.proto = proto

    def _serialize(self):
        return self.proto

    @classmethod
    def _deserialize(cls, value):
        return cls(*value)

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v: str):
        if not isinstance(v, str):
            raise TypeError("string required")
        binary = b64decode(v, validate=True)
        data = luba_msg_pb2.LubaMsg()
        data.ParseFromString(binary)
        return json_format.MessageToDict(data)


@dataclass
class DeviceProtobufMsgEventValue(DataClassORJSONMixin):
    content: str


@dataclass
class DeviceWarningEventValue(DataClassORJSONMixin):
    # TODO: enum for error codes
    # (see resources/res/values-en-rUS/strings.xml in APK)
    code: int


@dataclass
class DeviceConfigurationRequestValue(DataClassORJSONMixin):
    code: int
    bizId: str
    params: str


@dataclass
class DeviceNotificationEventCode(DataClassORJSONMixin):
    localTime: int
    code: str


@dataclass
class DeviceNotificationEventValue(DataClassORJSONMixin):
    data: str  # parsed to DeviceNotificationEventCode


@dataclass
class DeviceBizReqEventValue(DataClassORJSONMixin):
    bizType: str
    bizId: str
    params: str


@dataclass
class GeneralParams(DataClassORJSONMixin):
    group_id_list: Annotated[list[str], Alias("groupIdList")]
    group_id: Annotated[str, Alias("groupId")]
    category_key: Annotated[Literal["LawnMower", "Tracker"], Alias("categoryKey")]
    batch_id: Annotated[str, Alias("batchId")]
    gmt_create: Annotated[int, Alias("gmtCreate")]
    product_key: Annotated[str, Alias("productKey")]
    type: str
    device_name: Annotated[str, Alias("deviceName")]
    iot_id: Annotated[str, Alias("iotId")]
    check_level: Annotated[int, Alias("checkLevel")]
    namespace: str
    tenant_id: Annotated[str, Alias("tenantId")]
    name: str
    thing_type: Annotated[Literal["DEVICE"], Alias("thingType")]
    time: int
    tenant_instance_id: Annotated[str, Alias("tenantInstanceId")]
    value: Any

    # Optional fields
    identifier: str | None = None
    check_failed_data: Annotated[dict | None, Alias("checkFailedData")] = None
    _tenant_id: Annotated[str | None, Alias("_tenantId")] = None
    generate_time: Annotated[int | None, Alias("generateTime")] = None
    jmsx_delivery_count: Annotated[int | None, Alias("JMSXDeliveryCount")] = None
    qos: int | None = None
    request_id: Annotated[str | None, Alias("requestId")] = None
    _category_key: Annotated[str | None, Alias("_categoryKey")] = None
    device_type: Annotated[str | None, Alias("deviceType")] = None
    _trace_id: Annotated[str | None, Alias("_traceId")] = None

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True


@dataclass
class DeviceProtobufMsgEventParams(GeneralParams):
    identifier: Literal["device_protobuf_msg_event"]
    type: Literal["info"]
    value: DeviceProtobufMsgEventValue


@dataclass
class DeviceNotificationEventParams(GeneralParams):
    """Device notification event.

    {'data': '{"localTime":1725159492000,"code":"1002"}'},
    """

    identifier: Literal["device_notification_event", "device_information_event", "device_warning_code_event"]
    type: Literal["info"]
    value: DeviceNotificationEventValue


@dataclass
class DeviceBizReqEventParams(GeneralParams):
    identifier: Literal["device_biz_req_event"]
    type: Literal["info"]
    value: DeviceBizReqEventValue


@dataclass
class DeviceWarningEventParams(GeneralParams):
    identifier: Literal["device_warning_event"]
    type: Literal["alert"]
    value: DeviceWarningEventValue


@dataclass
class DeviceConfigurationRequestEvent(GeneralParams):
    type: Literal["info"]
    value: DeviceConfigurationRequestValue


@dataclass
class DeviceLogProgressEventParams(GeneralParams):
    identifier: Literal["device_log_progress_event"]
    type: Literal["info"]
    value: DeviceNotificationEventValue


@dataclass
class ThingEventMessage(DataClassORJSONMixin):
    method: Literal["thing.events", "thing.properties"]
    id: str
    params: (
        DeviceProtobufMsgEventParams
        | DeviceWarningEventParams
        | DeviceNotificationEventParams
        | DeviceLogProgressEventParams
        | DeviceBizReqEventParams
        | DeviceConfigurationRequestEvent
        | dict
    )
    version: Literal["1.0"]

    @classmethod
    def from_dicts(cls, payload: dict) -> "ThingEventMessage":
        """Deserialize payload JSON ThingEventMessage."""
        method = payload.get("method")
        event_id = payload.get("id")
        params_dict = payload.get("params", {})
        version = payload.get("version")

        identifier = params_dict.get("identifier")
        if identifier is None:
            """Request configuration event."""
            params_obj = DeviceConfigurationRequestEvent.from_dict(params_dict)
        elif identifier == "device_protobuf_msg_event":
            params_obj = DeviceProtobufMsgEventParams.from_dict(params_dict)
        elif identifier == "device_warning_event":
            params_obj = DeviceWarningEventParams.from_dict(params_dict)
        elif identifier == "device_biz_req_event":
            params_obj = DeviceBizReqEventParams.from_dict(params_dict)
        elif identifier == "device_log_progress_event":
            params_obj = DeviceLogProgressEventParams.from_dict(params_dict)
        elif identifier == "device_config_req_event":
            params_obj = payload.get("params", {})
        elif (
            identifier == "device_notification_event"
            or identifier == "device_warning_code_event"
            or identifier == "device_information_event"
        ):
            params_obj = DeviceNotificationEventParams.from_dict(params_dict)
        else:
            raise ValueError(f"Unknown identifier: {identifier} {params_dict}")

        return cls(method=method, id=event_id, params=params_obj, version=version)


@dataclass
class MammotionProtoMsgParams(DataClassORJSONMixin, SerializableType):
    value: DeviceProtobufMsgEventValue
    iot_id: str = ""
    product_key: str = ""
    device_name: str = ""

    @classmethod
    def _deserialize(cls, d: dict[str, Any]) -> "MammotionProtoMsgParams":
        """Override from_dict to allow dict manipulation before conversion."""
        proto: str = d["content"]

        return cls(value=DeviceProtobufMsgEventValue(content=proto))


@dataclass
class MammotionEventMessage(DataClassORJSONMixin):
    id: str
    version: str
    sys: dict
    params: MammotionProtoMsgParams
    method: str



"""RTK protobuf commands."""

from abc import ABC
from logging import getLogger
import time

from pymammotion.mammotion.commands.abstract_message import AbstractMessage
from pymammotion.proto import (
    AppToBaseMqttRtkT,
    BaseStation,
    LubaMsg,
    MsgAttr,
    MsgCmdType,
    MsgDevice,
    RequestBasestationInfoT,
)

logger = getLogger(__name__)


class MessageBasestation(AbstractMessage, ABC):
    def send_order_msg_basestation(self, driver) -> bytes:
        return LubaMsg(
            msgtype=MsgCmdType.BASESTATION,
            sender=MsgDevice.DEV_MOBILEAPP,
            rcver=self.get_msg_device(MsgCmdType.BASESTATION, MsgDevice.DEV_MAINCTL),
            msgattr=MsgAttr.REQ,
            timestamp=round(time.time() * 1000),
            seqs=self.seqs.increment_and_get() & 255,
            version=1,
            subtype=self.user_account,
            driver=driver,
        ).SerializeToString()

    def basestation_info(self) -> bytes:
        """Build and send a request to get basestation info (request_type=1)."""
        base = BaseStation(to_dev=RequestBasestationInfoT(request_type=1))
        return self.send_order_msg_basestation(base)

    def set_base_net_rtk_switch(self, rtk_switch: int) -> bytes:
        """Set RTK switch via app_to_base_mqtt_rtk_t."""
        base = BaseStation(app_to_base_mqtt_rtk_msg=AppToBaseMqttRtkT(rtk_switch=rtk_switch))
        return self.send_order_msg_basestation(base)



import math

import numpy as np
from numpy.typing import NDArray

from pymammotion.data.model.location import LocationPoint


class CoordinateConverter:
    """Converts between ENU (East-North-Up) and LLA (Latitude-Longitude-Altitude) coordinate systems.

    Uses WGS84 ellipsoid model for Earth coordinate transformations.
    """

    # WGS84 ellipsoid constants
    WGS84A: float = 6378137.0  # Semi-major axis (equatorial radius) in meters
    FLATTENING: float = 0.0033528106647474805  # WGS84 flattening factor

    def __init__(self, latitude_rad: float, longitude_rad: float, yaw_rad: float = 0.0) -> None:
        """Initialize the coordinate converter.

        Args:
            latitude_rad: Reference latitude in radians
            longitude_rad: Reference longitude in radians
            yaw_rad: Reference yaw angle in radians (default: 0.0)

        """
        # WGS84 ellipsoid parameters
        self.semi_major_axis: float = self.WGS84A
        self.flattening: float = self.FLATTENING
        self.semi_minor_axis: float = (1.0 - self.flattening) * self.WGS84A

        # Eccentricity parameters
        self.first_eccentricity_squared: float = (2.0 - self.flattening) * self.flattening
        self.eccentricity_ratio_squared: float = (1.0 - self.flattening) * (1.0 - self.flattening)
        self.second_eccentricity_squared: float = (self.semi_major_axis**2 - self.semi_minor_axis**2) / (
            self.semi_minor_axis**2
        )

        # Rotation matrix for coordinate transformation
        self.rotation_matrix: NDArray[np.float64] = np.zeros((3, 3), dtype=np.float64)

        # ECEF origin coordinates
        self.x0: float = 0.0
        self.y0: float = 0.0
        self.z0: float = 0.0

        # Yaw angle
        self.yaw: float = yaw_rad

        # Initialize with provided reference point
        self.set_init_lla(latitude_rad, longitude_rad, yaw_rad)

    def set_init_lla(self, latitude_rad: float, longitude_rad: float, yaw_rad: float = 0.0) -> None:
        """Set the initial LLA reference point and yaw angle.

        Args:
            latitude_rad: Reference latitude in radians
            longitude_rad: Reference longitude in radians
            yaw_rad: Reference yaw angle in radians (default: 0.0)

        """
        self.yaw = yaw_rad

        sin_lat = math.sin(latitude_rad)
        cos_lat = math.cos(latitude_rad)
        sin_lon = math.sin(longitude_rad)
        cos_lon = math.cos(longitude_rad)

        # Radius of curvature in the prime vertical
        N = self.semi_major_axis / math.sqrt(1.0 - self.first_eccentricity_squared * (sin_lat**2))

        # Calculate ECEF origin coordinates (altitude = 0)
        horizontal_distance = N * cos_lat
        self.x0 = horizontal_distance * cos_lon
        self.y0 = horizontal_distance * sin_lon
        self.z0 = self.eccentricity_ratio_squared * N * sin_lat

        # Build rotation matrix (ECEF to ENU)
        self.rotation_matrix[0, 0] = -sin_lon
        self.rotation_matrix[0, 1] = cos_lon
        self.rotation_matrix[0, 2] = 0.0

        self.rotation_matrix[1, 0] = -cos_lon * sin_lat
        self.rotation_matrix[1, 1] = -sin_lon * sin_lat
        self.rotation_matrix[1, 2] = cos_lat

        self.rotation_matrix[2, 0] = cos_lon * cos_lat
        self.rotation_matrix[2, 1] = sin_lon * cos_lat
        self.rotation_matrix[2, 2] = sin_lat

    def enu_to_lla(self, east: float, north: float) -> LocationPoint:
        """Convert ENU (East-North-Up) coordinates to LLA (Latitude-Longitude-Altitude).

        Args:
            east: East coordinate in meters
            north: North coordinate in meters

        Returns:
            Point with latitude and longitude in degrees

        """
        # Transform ENU to ECEF (Earth-Centered, Earth-Fixed) coordinates
        # using rotation matrix and origin offset
        ecef_x = self.rotation_matrix[0, 0] * north + self.rotation_matrix[1, 0] * east + self.x0
        ecef_y = self.rotation_matrix[0, 1] * north + self.rotation_matrix[1, 1] * east + self.y0
        ecef_z = self.rotation_matrix[0, 2] * north + self.rotation_matrix[1, 2] * east + self.z0

        # Calculate horizontal distance from Earth's axis
        horizontal_distance = math.hypot(ecef_x, ecef_y)

        # Initial latitude estimate using simplified formula
        initial_latitude = math.atan2(self.semi_major_axis * ecef_z, self.semi_minor_axis * horizontal_distance)

        sin_initial_lat = math.sin(initial_latitude)
        cos_initial_lat = math.cos(initial_latitude)

        # Calculate longitude (straightforward conversion from ECEF)
        longitude_deg = math.degrees(math.atan2(ecef_y, ecef_x))

        # Calculate precise latitude using iterative correction
        # This accounts for Earth's ellipsoidal shape
        latitude_rad = math.atan2(
            ecef_z + self.second_eccentricity_squared * self.semi_minor_axis * (sin_initial_lat**3),
            horizontal_distance - self.first_eccentricity_squared * self.semi_major_axis * (cos_initial_lat**3),
        )
        latitude_deg = math.degrees(latitude_rad)

        return LocationPoint(latitude=latitude_deg, longitude=longitude_deg)

    def lla_to_enu(self, longitude_deg: float, latitude_deg: float) -> list[float]:
        """Convert LLA (Latitude-Longitude-Altitude) to ENU (East-North-Up) coordinates.

        Args:
            longitude_deg: Longitude in degrees
            latitude_deg: Latitude in degrees

        Returns:
            List of [east, north] coordinates in meters

        """
        # Convert to radians
        lat_rad = math.radians(latitude_deg)
        lon_rad = math.radians(longitude_deg)

        sin_lat = math.sin(lat_rad)
        cos_lat = math.cos(lat_rad)
        sin_lon = math.sin(lon_rad)
        cos_lon = math.cos(lon_rad)

        # Calculate radius of curvature
        N = self.semi_major_axis / math.sqrt(1.0 - self.first_eccentricity_squared * (sin_lat**2))

        # Convert to ECEF (altitude = 0)
        horizontal = N * cos_lat
        ecef_x = horizontal * cos_lon
        ecef_y = horizontal * sin_lon
        ecef_z = self.eccentricity_ratio_squared * N * sin_lat

        # Translate to origin
        dx = ecef_x - self.x0
        dy = ecef_y - self.y0
        dz = ecef_z - self.z0

        # Rotate to ENU frame
        east = self.rotation_matrix[0, 0] * dx + self.rotation_matrix[0, 1] * dy + self.rotation_matrix[0, 2] * dz
        north = self.rotation_matrix[1, 0] * dx + self.rotation_matrix[1, 1] * dy + self.rotation_matrix[1, 2] * dz

        # Apply yaw rotation (inverse)
        rotated_east = math.cos(-self.yaw) * east - math.sin(-self.yaw) * north
        rotated_north = math.sin(-self.yaw) * east + math.cos(-self.yaw) * north

        return [round(rotated_east, 3), round(rotated_north, 3)]

    def get_transform_yaw_with_yaw(self, yaw_degrees: float) -> float:
        """Transform a yaw angle from global coordinates to local coordinates.

        This applies the inverse of the reference yaw rotation to convert a global
        heading angle into the local coordinate frame.

        Args:
            yaw_degrees: Input yaw angle in degrees

        Returns:
            Transformed yaw angle in degrees

        """
        # Convert input angle to radians
        yaw_rad = math.radians(yaw_degrees)

        # Apply inverse rotation: -self.yaw
        inverse_yaw = -self.yaw

        # Using angle addition formula: atan2(sin(a+b), cos(a+b))
        # where a = inverse_yaw, b = yaw_rad
        sin_sum = math.sin(inverse_yaw) * math.cos(yaw_rad) + math.cos(inverse_yaw) * math.sin(yaw_rad)
        cos_sum = math.cos(inverse_yaw) * math.cos(yaw_rad) - math.sin(inverse_yaw) * math.sin(yaw_rad)

        # Calculate resulting angle and convert back to degrees
        result_rad = math.atan2(sin_sum, cos_sum)
        result_degrees = math.degrees(result_rad)

        return result_degrees

    def get_angle_yaw(self) -> float:
        """Get the current yaw angle in degrees.

        Returns:
            Yaw angle in degrees

        """
        return math.degrees(self.yaw)

    def get_yaw(self) -> float:
        """Get the current yaw angle in radians.

        Returns:
            Yaw angle in radians

        """
        return self.yaw

    def set_yaw(self, yaw_rad: float) -> None:
        """Set the yaw angle.

        Args:
            yaw_rad: Yaw angle in radians

        """
        self.yaw = yaw_rad

    def set_yaw_degrees(self, yaw_degrees: float) -> None:
        """Set the yaw angle from degrees.

        Args:
            yaw_degrees: Yaw angle in degrees

        """
        self.yaw = math.radians(yaw_degrees)


# Usage example
if __name__ == "__main__":
    # Initialize converter with reference point
    converter = CoordinateConverter(
        latitude_rad=math.radians(40.0), longitude_rad=math.radians(-105.0), yaw_rad=math.radians(45.0)
    )

    # Convert ENU to LLA
    point = converter.enu_to_lla(east=100.0, north=200.0)
    print(f"Latitude: {point.latitude}, Longitude: {point.longitude}")

    # Convert LLA to ENU
    enu = converter.lla_to_enu(longitude_deg=-105.0, latitude_deg=40.0)
    print(f"East: {enu[0]}, North: {enu[1]}")

    # Transform yaw angle
    transformed_yaw = converter.get_transform_yaw_with_yaw(90.0)
    print(f"Transformed yaw: {transformed_yaw}°")



from dataclasses import dataclass

from mashumaro.mixins.orjson import DataClassORJSONMixin

from pymammotion.data.mqtt.properties import Items


@dataclass
class ThingPropertiesResponse(DataClassORJSONMixin):
    code: int
    data: Items | None
    id: str | None = None



"""Mower device with Bluetooth LE connectivity."""

import logging
from typing import Any

from bleak import BLEDevice

from pymammotion.aliyun.model.dev_by_account_response import Device
from pymammotion.data.mower_state_manager import MowerStateManager
from pymammotion.mammotion.devices.mammotion_bluetooth import MammotionBaseBLEDevice
from pymammotion.mammotion.devices.mower_device import MammotionMowerDevice

_LOGGER = logging.getLogger(__name__)


class MammotionMowerBLEDevice(MammotionBaseBLEDevice, MammotionMowerDevice):
    """Mower device with BLE connectivity and map synchronization."""

    def __init__(
        self,
        state_manager: MowerStateManager,
        cloud_device: Device,
        device: BLEDevice,
        interface: int = 0,
        **kwargs: Any,
    ) -> None:
        """Initialize MammotionMowerBLEDevice.

        Uses multiple inheritance to combine:
        - MammotionBaseBLEDevice: BLE communication
        - MammotionMowerDevice: Map sync callbacks
        """
        # Initialize base BLE device (which also initializes MammotionBaseDevice)
        MammotionBaseBLEDevice.__init__(self, state_manager, cloud_device, device, interface, **kwargs)
        # Set up mower-specific BLE callbacks
        self._state_manager.ble_gethash_ack_callback = self.datahash_response
        self._state_manager.ble_get_commondata_ack_callback = self.commdata_response
        self._state_manager.ble_get_plan_callback = self.plan_callback

    def __del__(self) -> None:
        """Cleanup subscriptions and callbacks."""
        # Clean up mower-specific callbacks
        if hasattr(self, "_state_manager"):
            self._state_manager.ble_gethash_ack_callback = None
            self._state_manager.ble_get_commondata_ack_callback = None
            self._state_manager.ble_get_plan_callback = None
        # Call parent cleanup
        super().__del__()



"""RTK device with cloud MQTT connectivity."""

import asyncio
from collections.abc import Awaitable, Callable
import logging
from typing import Any

from pymammotion.aliyun.model.dev_by_account_response import Device
from pymammotion.data.model.device import RTKDevice
from pymammotion.data.mqtt.properties import ThingPropertiesMessage
from pymammotion.data.mqtt.status import ThingStatusMessage
from pymammotion.mammotion.commands.mammotion_command import MammotionCommand
from pymammotion.mammotion.devices.mammotion_cloud import MammotionCloud
from pymammotion.mammotion.devices.rtk_device import MammotionRTKDevice

_LOGGER = logging.getLogger(__name__)


class MammotionRTKCloudDevice(MammotionRTKDevice):
    """RTK device with cloud connectivity - simpler than mowers, no map sync."""

    def __init__(self, mqtt: MammotionCloud, cloud_device: Device, rtk_state: RTKDevice) -> None:
        """Initialize MammotionRTKCloudDevice.
        :rtype: None
        """
        super().__init__(cloud_device, rtk_state)
        self.stopped = False
        self.on_ready_callback: Callable[[], Awaitable[None]] | None = None
        self.loop = asyncio.get_event_loop()
        self._mqtt = mqtt
        self.iot_id = cloud_device.iot_id
        self.device = cloud_device
        self._commands: MammotionCommand = MammotionCommand(
            cloud_device.device_name,
            int(mqtt.cloud_client.mammotion_http.response.data.userInformation.userAccount),
        )
        # Subscribe to MQTT events for this device
        self._mqtt.mqtt_properties_event.add_subscribers(self._parse_message_properties_for_device)
        self._mqtt.mqtt_status_event.add_subscribers(self._parse_message_status_for_device)
        self._mqtt.on_ready_event.add_subscribers(self.on_ready)
        self._mqtt.on_disconnected_event.add_subscribers(self.on_disconnect)
        self._mqtt.on_connected_event.add_subscribers(self.on_connect)

    def __del__(self) -> None:
        """Cleanup subscriptions."""
        if hasattr(self, "_mqtt"):
            self._mqtt.on_ready_event.remove_subscribers(self.on_ready)
            self._mqtt.on_disconnected_event.remove_subscribers(self.on_disconnect)
            self._mqtt.on_connected_event.remove_subscribers(self.on_connect)
            self._mqtt.mqtt_properties_event.remove_subscribers(self._parse_message_properties_for_device)
            self._mqtt.mqtt_status_event.remove_subscribers(self._parse_message_status_for_device)

    @property
    def command_sent_time(self) -> float:
        return self._mqtt.command_sent_time

    @property
    def mqtt(self) -> MammotionCloud:
        return self._mqtt

    async def on_ready(self) -> None:
        """Callback for when MQTT is subscribed to events."""
        if self.stopped:
            return
        if self.on_ready_callback:
            await self.on_ready_callback()

    async def on_disconnect(self) -> None:
        """Callback for when MQTT disconnects."""
        self._mqtt.disconnect()

    async def on_connect(self) -> None:
        """Callback for when MQTT connects."""

    def stop(self) -> None:
        """Stop all tasks and disconnect."""
        self.stopped = True

    async def start(self) -> None:
        """Start the device connection."""
        self.stopped = False
        if not self.mqtt.is_connected():
            loop = asyncio.get_running_loop()
            await loop.run_in_executor(None, self.mqtt.connect_async)

    async def queue_command(self, key: str, **kwargs: Any) -> None:
        """Queue a command to the RTK device."""
        _LOGGER.debug("Queueing command: %s", key)
        future = asyncio.Future()
        command_bytes = getattr(self._commands, key)(**kwargs)
        await self._mqtt.command_queue.put((self.iot_id, key, command_bytes, future))
        try:
            await future
        except asyncio.CancelledError:
            """Try again once."""
            future = asyncio.Future()
            await self._mqtt.command_queue.put((self.iot_id, key, command_bytes, future))

    async def _parse_message_properties_for_device(self, event: ThingPropertiesMessage) -> None:
        """Parse property messages for this RTK device."""
        if event.params.iot_id != self.iot_id:
            return
        # RTK devices have simpler properties - update as needed
        _LOGGER.debug("RTK properties update: %s", event)

    async def _parse_message_status_for_device(self, status: ThingStatusMessage) -> None:
        """Parse status messages for this RTK device."""
        if status.params.iot_id != self.iot_id:
            return
        # Update online status
        self._rtk_device.online = True
        _LOGGER.debug("RTK status update: %s", status)

    async def _ble_sync(self) -> None:
        """RTK devices don't use BLE sync in the same way as mowers."""
